"""
Virtualization test - cgroup related utility functions for libvirt

:copyright: 2019 Red Hat Inc.
"""
import os
import logging
import re

from avocado.utils import process

from virttest import virsh
from virttest.staging import utils_cgroup

VIRSH_BLKIOTUNE_OUTPUT_MAPPING = {"weight": "weight",
                                  "device_read_iops_sec": "riops",
                                  "device_write_iops_sec": "wiops",
                                  "device_read_bytes_sec": "rbps",
                                  "device_write_bytes_sec": "wbps"}
CGROUP_V1_BLKIO_FILE_MAPPING = {"weight": "blkio.bfq.weight",
                                "wiops": "blkio.throttle.write_iops_device",
                                "riops": "blkio.throttle.read_iops_device",
                                "rbps": "blkio.throttle.read_bps_device",
                                "wbps": "blkio.throttle.write_bps_device"}
CGROUP_V2_BLKIO_FILE_MAPPING = {"weight": "io.bfq.weight",
                                "wiops": "io.max",
                                "riops": "io.max",
                                "rbps": "io.max",
                                "wbps": "io.max"}
CGROUP_V1_MEM_FILE_MAPPING = {"hard_limit": "memory.limit_in_bytes",
                              "soft_limit": "memory.soft_limit_in_bytes",
                              "swap_hard_limit": "memory.memsw.limit_in_bytes"}
CGROUP_V2_MEM_FILE_MAPPING = {"hard_limit": "memory.max",
                              "soft_limit": "memory.high",
                              "swap_hard_limit": "memory.swap.max"}
CGROUP_V1_SCHEDINFO_FILE_MAPPING = {"cpu_shares": "cpu.shares",
                                    "vcpu_period": "<vcpuX>/cpu.cfs_period_us",
                                    "vcpu_quota": "<vcpuX>/cpu.cfs_quota_us",
                                    "emulator_period": "emulator/cpu.cfs_period_us",
                                    "emulator_quota": "emulator/cpu.cfs_quota_us",
                                    "global_period": "cpu.cfs_period_us",
                                    "global_quota": "cpu.cfs_quota_us",
                                    "iothread_period": "<iothreadX>/cpu.cfs_period_us",
                                    "iothread_quota": "<iothreadX>/cpu.cfs_quota_us"}
CGROUP_V2_SCHEDINFO_FILE_MAPPING = {"cpu_shares": "cpu.weight",
                                    "vcpu_period": "<vcpuX>/cpu.max",
                                    "vcpu_quota": "<vcpuX>/cpu.max",
                                    "emulator_period": "emulator/cpu.max",
                                    "emulator_quota": "emulator/cpu.max",
                                    "global_period": "cpu.max",
                                    "global_quota": "cpu.max",
                                    "iothread_period": "<iothreadX>/cpu.max",
                                    "iothread_quota": "<iothreadX>/cpu.max"}


#cgroup related functions
class CgroupTest(object):

    """Class for libvirt cgroup related test"""

    __vm_pid = ""

    def __init__(self, vm_pid):
        self.__vm_pid = vm_pid

    def is_cgroup_v2_enabled(self):
        """
        Check if cgroup v2 enabled on host

        :return: True means cgroup v2 enabled
                 False means not
        """
        cmd = "cat /proc/mounts | grep cgroup2"
        cmd_result = process.run(cmd, ignore_status=True, shell=True)
        if cmd_result.exit_status:
            return False
        return True

    def get_cgroup_path(self, controller=None):
        """
        Get specific cgroup controller's root path

        :params controller: The cgroup controller, used for cgroup v1. For
                            cgroup v2 this param will be ignored since all
                            controllers are in the same dir
        :return: The path to the cgroup controller
        """
        cgroup_path = ""
        if self.is_cgroup_v2_enabled():
            vm_proc_cgroup_path = "/proc/%s/cgroup" % self.__vm_pid
            with open('/proc/mounts', 'r') as mnts:
                cg_mount_point = re.findall(r"\s(\S*cgroup)\s", mnts.read())
            with open(vm_proc_cgroup_path, 'r') as vm_cg_file:
                cg_vm_scope = re.findall(r"\S*::(\S*)", vm_cg_file.read())
            cgroup_path = os.path.join(cg_mount_point[0],
                                       cg_vm_scope[0].strip("/"))
            if "emulator" in cgroup_path:
                cgroup_path += "/.."
        else:
            cgroup_path = utils_cgroup.resolve_task_cgroup_path(
                    int(self.__vm_pid), controller)
        if not os.path.exists(cgroup_path):
            logging.error("cgroup path '%s' doesn't exist" % cgroup_path)
            return None
        return cgroup_path

    def __get_cpu_subdirs(self, controller_path=None, dir_keyword=None):
        """
        Search and return the sub dirs of the cpu controllers by keyword

        :param controller_path: The path of the cpu controller
        :param dir_keyword: The keyword of the sub dirs. Normally it could be
                            "vcpu", "emulator", "iothread"
        :return: The list of the sub dir names
        """
        dir_names = []
        for filename in os.listdir(controller_path):
            if dir_keyword in filename:
                dir_names.append(filename)
        if not dir_names and "iothread" in dir_keyword:
            logging.debug("No sub dirs found with keyword: '%s'. "
                          "Pls check if you've executed virsh cmd "
                          "'iothreadadd'.", dir_keyword)
            return None
        return sorted(dir_names)

    def __get_standardized_cgroup1_info(self, virsh_cmd=None):
        """
        Get the cgroup info on a cgroupv1 enabled system, and standardize it to
        a dict

        :param virsh_cmd: The virsh cmd used. This is to judge which cgroup
                          info to get. Such as, when virsh cmd is 'blkiotune',
                          the blkio related cgroup info will be returned
        :return: A dict containing the cgroup info
        """
        standardized_cgroup_info = {}
        if virsh_cmd == "blkiotune":
            cgroup_path = self.get_cgroup_path("blkio")
            dev_init_dict = {"rbps": "max", "wbps": "max", "riops": "max",
                             "wiops": "max"}
            dev_list = []
            for cg_key, cg_file_name in list(CGROUP_V1_BLKIO_FILE_MAPPING.items()):
                with open(os.path.join(cgroup_path, cg_file_name), 'r') as cg_file:
                    if cg_key in ["weight"]:
                        standardized_cgroup_info[cg_key] = cg_file.read().strip()
                    if cg_key in ["rbps", "wbps", "riops", "wiops"]:
                        for line in cg_file.readlines():
                            dev_info = line.strip().split()
                            dev_num = dev_info[0]
                            dev_cg_value = dev_info[1]
                            if dev_num not in dev_list:
                                standardized_cgroup_info[dev_num] = dev_init_dict.copy()
                            standardized_cgroup_info[dev_num][cg_key] = dev_cg_value
                            dev_list.append(dev_num)
        elif virsh_cmd == "memtune":
            cgroup_path = self.get_cgroup_path("memory")
            max_mem_value = "9223372036854771712"
            for cg_key, cg_file_name in list(CGROUP_V1_MEM_FILE_MAPPING.items()):
                with open(os.path.join(cgroup_path, cg_file_name), 'r') as cg_file:
                    cg_file_value = cg_file.read().strip()
                    if cg_file_value == max_mem_value:
                        cg_file_value = "max"
                    standardized_cgroup_info[cg_key] = cg_file_value
        elif virsh_cmd == "schedinfo":
            cgroup_path = self.get_cgroup_path("cpu,cpuacct") + "/.."
            max_cpu_value = "-1"
            vcpu_dirs = self.__get_cpu_subdirs(cgroup_path, "vcpu")
            iothread_dirs = self.__get_cpu_subdirs(cgroup_path, "iothread")
            for cg_key, cg_file_name in list(CGROUP_V1_SCHEDINFO_FILE_MAPPING.items()):
                if "<vcpuX>" in cg_file_name:
                    cg_file_name = cg_file_name.replace("<vcpuX>", vcpu_dirs[0])
                if "<iothreadX>" in cg_file_name:
                    if iothread_dirs:
                        cg_file_name = cg_file_name.replace("<iothreadX>", iothread_dirs[0])
                    else:
                        continue
                with open(os.path.join(cgroup_path, cg_file_name), 'r') as cg_file:
                    cg_file_value = cg_file.read().strip()
                    if cg_file_value == max_cpu_value:
                        cg_file_value = "max"
                    standardized_cgroup_info[cg_key] = cg_file_value
        else:
            logging.error("You've provided a wrong virsh cmd: %s", virsh_cmd)
        return standardized_cgroup_info

    def __get_standardized_cgroup2_info(self, virsh_cmd=None):
        """
        Get the cgroup info on a cgroupv2 enabled system, and standardize it to
        a dict

        :param virsh_cmd: The virsh cmd used. This is to judge which cgroup
                          info to get. Such as, when virsh cmd is 'blkiotune',
                          the blkio related cgroup info will be returned
        :return: A dict containing the cgroup info
        """
        standardized_cgroup_info = {}
        cgroup_path = self.get_cgroup_path()
        if virsh_cmd == "blkiotune":
            weight_file_name = CGROUP_V2_BLKIO_FILE_MAPPING["weight"]
            iomax_file_name = CGROUP_V2_BLKIO_FILE_MAPPING["wiops"]
            path_to_weight = os.path.join(cgroup_path, weight_file_name)
            with open(path_to_weight, 'r') as weight_file:
                weight_value = re.search(r'\d+', weight_file.read())
                if weight_value:
                    weight_value = weight_value.group()
                standardized_cgroup_info["weight"] = weight_value
            path_to_iomax = os.path.join(cgroup_path, iomax_file_name)
            with open(path_to_iomax, 'r') as iomax_file:
                iomax_info = iomax_file.readlines()
                for line in iomax_info:
                    dev_iomax_info = line.strip().split()
                    dev_iomax_dict = {}
                    dev_num = dev_iomax_info[0]
                    for i in range(1, len(dev_iomax_info)):
                        key, value = dev_iomax_info[i].split("=")
                        dev_iomax_dict[key] = value
                    standardized_cgroup_info[dev_num] = dev_iomax_dict
        elif virsh_cmd == "memtune":
            for cg_key, cg_file_name in list(CGROUP_V2_MEM_FILE_MAPPING.items()):
                with open(os.path.join(cgroup_path, cg_file_name), 'r') as cg_file:
                    cg_file_value = cg_file.read().strip()
                    standardized_cgroup_info[cg_key] = cg_file_value
        elif virsh_cmd == "schedinfo":
            vcpu_dirs = self.__get_cpu_subdirs(cgroup_path, "vcpu")
            iothread_dirs = self.__get_cpu_subdirs(cgroup_path, "iothread")
            for cg_key, cg_file_name in list(CGROUP_V2_SCHEDINFO_FILE_MAPPING.items()):
                if "<vcpuX>" in cg_file_name:
                    cg_file_name = cg_file_name.replace("<vcpuX>", vcpu_dirs[0])
                if "<iothreadX>" in cg_file_name:
                    if iothread_dirs:
                        cg_file_name = cg_file_name.replace("<iothreadX>", iothread_dirs[0])
                    else:
                        continue
                with open(os.path.join(cgroup_path, cg_file_name), 'r') as cg_file:
                    list_index = 0
                    cg_file_values = cg_file.read().strip().split()
                    if "period" in cg_key:
                        list_index = 1
                    standardized_cgroup_info[cg_key] = cg_file_values[list_index]
        else:
            logging.error("You've provided a wrong virsh cmd: %s", virsh_cmd)
        return standardized_cgroup_info

    def get_standardized_cgroup_info(self, virsh_cmd=None):
        """
        Get the cgroup info and standardize it to a dict

        :param virsh_cmd: The virsh cmd used. This is to judge which cgroup
                          info to get. Such as, when virsh cmd is 'blkiotune',
                          the blkio related cgroup info will be returned
        :return: A dict containing the cgroup info
        """
        if self.is_cgroup_v2_enabled():
            return self.__get_standardized_cgroup2_info(virsh_cmd)
        else:
            return self.__get_standardized_cgroup1_info(virsh_cmd)

    def get_virsh_output_dict(self, vm_name=None, virsh_cmd=None):
        """
        Get the virsh cmd output as a dict

        :param vm_name: Name of the vm
        :param virsh_cmd: Name of the virsh cmd

        :return: The virsh cmd output, as a dict
        """
        if virsh_cmd == "memtune":
            func = virsh.memtune_list
        elif virsh_cmd == "blkiotune":
            func = virsh.blkiotune
        elif virsh_cmd == "schedinfo":
            func = virsh.schedinfo
        else:
            logging.error("There is no virsh cmd '%s'", virsh_cmd)
            return None
        result = func(vm_name, ignore_status=True)
        output = result.stdout_text.strip()
        output_list = output.splitlines()
        output_dict = {}
        for output_line in output_list:
            output_info = output_line.split(":")
            output_param = output_info[0].strip()
            if len(output_info) == 1:
                output_value = ""
            else:
                output_value = output_info[1].strip()
            output_dict[output_param] = output_value
        return output_dict

    def __get_dev_major_minor(self, dev_path="/dev/sda"):
        """
        Get the device 'major:minor' number

        :param dev_path: The path to the device
        """
        if not os.path.exists(dev_path):
            logging.debug("device '%s' not existing", dev_path)
            return None
        dev = os.stat(dev_path)
        return "%s:%s" % (os.major(dev.st_rdev), os.minor(dev.st_rdev))

    def get_standardized_virsh_info(self, virsh_cmd=None, virsh_dict=None):
        """
        Get and standardize the info of cgroup related virsh cmd's output

        :param virsh_cmd: The virsh cmd used
        :param virsh_dict: The dict containing the virsh cmd output
        :return: Standardized info of the virsh cmd output, as a dict
        """
        standardized_virsh_output_info = {}
        if virsh_cmd == "blkiotune":
            virsh_output_mapping = VIRSH_BLKIOTUNE_OUTPUT_MAPPING.copy()
            dev_list = []
            dev_init_dict = {"rbps": "max", "wbps": "max", "riops": "max",
                             "wiops": "max"}
            for io_item, io_item_value in list(virsh_dict.items()):
                if io_item in ["weight"]:
                    standardized_virsh_output_info[io_item] = io_item_value
                elif io_item in list(virsh_output_mapping.keys()) and io_item_value:
                    io_value_list = io_item_value.split(",")
                    for i in range(len(io_value_list)):
                        if "dev" in io_value_list[i]:
                            dev_num = self.__get_dev_major_minor(io_value_list[i])
                            if dev_num not in dev_list:
                                standardized_virsh_output_info[dev_num] = dev_init_dict.copy()
                                dev_list.append(dev_num)
                            standardized_virsh_output_info[dev_num][virsh_output_mapping[io_item]] = io_value_list[i+1]
        elif virsh_cmd == "memtune":
            standardized_virsh_output_info = {"hard_limit": "max",
                                              "soft_limit": "max",
                                              "swap_hard_limit": "max"}
            for mem_item, mem_item_value in list(virsh_dict.items()):
                if mem_item_value in ["unlimited"]:
                    standardized_virsh_output_info[mem_item] = "max"
                elif mem_item_value.isdigit():
                    standardized_virsh_output_info[mem_item] = str(int(mem_item_value) * 1024)
                else:
                    standardized_virsh_output_info[mem_item] = mem_item_value
                    logging.debug("memtune: the value '%s' for '%s' is "
                                  "new to us, pls check.",
                                  mem_item_value, mem_item)
        elif virsh_cmd == "schedinfo":
            for schedinfo_item, schedinfo_value in list(virsh_dict.items()):
                if schedinfo_item.lower() in ["scheduler"]:
                    # no need to check scheduler type, it's fixed for qemu
                    continue
                if "quota" in schedinfo_item:
                    if schedinfo_value in ["-1", "18446744073709551"]:
                        standardized_virsh_output_info[schedinfo_item] = "max"
                        continue
                standardized_virsh_output_info[schedinfo_item] = schedinfo_value
        else:
            logging.error("You've provided an unsupported virsh cmd: %s",
                          virsh_cmd)
            return None
        return standardized_virsh_output_info

    def get_standardized_virsh_output_by_name(self, vm_name=None, virsh_cmd=None):
        """
        Get the standardized output of a vm with a certain virsh cmd

        :param vm_name: The name of the vm
        :param virsh_cmd: The virsh cmd to get the output
        :return: The standardized dict of the virsh cmd output
        """
        virsh_output_dict = self.get_virsh_output_dict(vm_name, virsh_cmd)
        return self.get_standardized_virsh_info(virsh_cmd, virsh_output_dict)



import logging
import os
import re
import shutil

from avocado.core import exceptions
from avocado.utils import process

from . import utils_net


class Machine(object):

    def __init__(self, vm=None, src=None):
        self.vm = vm
        self.session = None
        self.runner = utils_net.local_runner
        self.runner_status = utils_net.local_runner_status
        self.bg_runner = process.SubProcess
        self.src = src
        self.addrs = None
        if vm:
            self.session = vm.wait_for_login()
            self.runner = self.session.cmd
            self.runner_status = self.session.cmd_status
            self.bg_runner = self.session.sendline
            self.src = os.path.join("/", "tmp", "src")

    def is_virtual(self):
        """
        :return: True when Machine is virtual.
        """
        return self.vm is not None

    def cmd(self, cmd, timeout=60):
        """
        Return outpu of command.
        """
        return self.runner(cmd, timeout=timeout)

    def cmd_state(self, cmd, timeout=60):
        """
        Return status of command.
        """
        return self.runner_status(cmd, timeout=timeout)

    def cmd_in_src(self, cmd, timeout=60):
        cmd = os.path.join(self.src, cmd)
        return self.runner(cmd, timeout=timeout)

    def fill_addrs(self):
        self.addrs = utils_net.get_net_if_and_addrs(self.runner)
        if self.vm:
            self.vm.fill_addrs(self.addrs)

    def get_linkv6_addr(self, ifname):
        """
        Get IPv6 address with link range.

        :param ifname: String or int. Int could be used only for virt Machine.
        :return: IPv6 link address.
        """
        if self.is_virtual() and type(ifname) is int:
            ifname = self.vm.virtnet[ifname].g_nic_name
        return utils_net.ipv6_from_mac_addr(self.addrs[ifname]['mac'])

    def ping(self, dst, iface=None, count=1, vlan=0, ipv=None):
        """
        Ping destination.
        """
        command = "ping"
        if ipv is None or ":" in dst:
            command = "ping6"
            if iface is None:
                raise exceptions.TestError(
                    "For ipv6 ping, interface can't be None")
            if self.vm:
                iface = self.get_if_vlan_name(
                    self.vm.virtnet[iface].g_nic_name,
                    vlan)
            else:
                iface = self.get_if_vlan_name(iface, vlan)

        if dst is not None:
            command += " %s" % dst
        if iface is not None:
            command += " -I %s" % iface
        if count is not None:
            command += " -c %s" % count

        self.runner(command)

    def add_vlan_iface(self, iface, vlan_id):
        """
        Add vlan link for interface

        :param iface: Interface on which should be added vlan.
        :param vlan_id: Id of vlan.
        """
        self.cmd("ip link add link %s name %s-vl%s type vlan id %s" %
                 (iface, iface, vlan_id, vlan_id))

    def del_vlan_iface(self, iface, vlan_id):
        """
        Del vlan link for interface

        :param iface: Interface from which should be deleted vlan.
        :param vlan_id: Id of vlan.
        """
        self.cmd("ip link del %s" % (iface))

    def bring_iface_up(self, iface):
        """
        Bring interface up
        """
        self.cmd("ip link set %s up" % (iface))

    def bring_iface_down(self, iface):
        """
        Bring interface up
        """
        self.cmd("ip link set %s down" % (iface))

    def get_vlans_ifname(self):
        """
        Return vlans interface name.

        :return: dict of {"ifname": [(vlanid, ifname),(...)],...}
        """
        ret = dict()
        vlans = self.cmd("cat /proc/net/vlan/config")
        v = re.findall("^(\S+)\s*\|\s*(\S+)\s*\|\s*(\S+)\s*$",
                       vlans, re.MULTILINE)
        for vl_ifname, vl_id, ifname in v:
            if ifname in ret:
                ret[ifname][int(vl_id)] = vl_ifname
            else:
                ret[ifname] = {int(vl_id): vl_ifname}

        return ret

    def get_if_vlan_name(self, ifname, vlan_id=0):
        if vlan_id == 0:
            return ifname

        vlans = self.get_vlans_ifname()
        if ifname in vlans:
            if vlan_id in vlans[ifname]:
                return vlans[ifname][vlan_id]
            else:
                raise utils_net.VlanError(ifname,
                                          "Interface %s has no vlan with"
                                          " id %s" % (ifname, vlan_id))
        else:
            raise utils_net.VlanError(ifname,
                                      "Interface %s has no vlans" % (ifname))

    def prepare_directory(self, path, cleanup=False):
        """
        Prepare dest directory. Create if directory not exist.

        :param path: Path to directory
        :param cleanup: It true clears the contents of directory.
        """
        if self.cmd_state("test -x %s" % path):
            self.cmd("mkdir -p %s" % path)

        if cleanup:
            self.cmd("rm -rf %s" % (os.path.join(path, "*")))

    def copy_to(self, src, dst):
        if self.vm:
            self.vm.copy_files_to(src, dst)
        else:
            shutil.copy(src, dst)

    def compile_autotools_app_tar(self, path, package_name):
        """
        Compile app on machine in src dir.

        :param path: Path where shoule be program compiled.
        :param dst_dir: Installation path.
        """
        logging.debug("Install %s to %s.", package_name, self.src)
        self.prepare_directory(self.src)

        pack_dir = None
        if package_name.endswith("tar.gz"):
            pack_dir = package_name[0:-len(".tar.gz")]
            unpack_cmd = ("tar -xvzf %s; cd %s;" % (package_name, pack_dir))
        elif package_name.endswith("tgz"):
            pack_dir = package_name[0:-len(".tgz")]
            unpack_cmd = ("tar -xvzf %s; cd %s;" % (package_name, pack_dir))
        elif package_name.endswith(("tar.bz2")):
            pack_dir = package_name[0:-len(".tar.br2")]
            unpack_cmd = ("tar -xvjf %s; cd %s;" % (package_name, pack_dir))

        self.copy_to(os.path.join(path, package_name), self.src)
        self.cmd("sync")
        self.cmd("cd %s; %s ./configure && make;" % (self.src, unpack_cmd),
                 timeout=240)
        self.cmd("sync")

    def __getattr__(self, name):
        if self.vm:
            try:
                return self.vm.__getattribute__(name)
            except AttributeError:
                return self.session.__getattribute__(name)
        raise AttributeError("Cannot find attribute %s in class" % name)



"""
Library to perform iptables configuration for virt test.
"""
import logging

from aexpect import remote

from avocado.utils import process
from avocado.core import exceptions


class Iptables(object):
    """
    class to handle all iptables configurations related methods
    """

    @classmethod
    def setup_or_cleanup_iptables_rules(cls, rules, params=None,
                                        cleanup=False):
        """
        Setup or cleanup for iptable rules, it can be locally or remotely

        :param rules: list of rules
        :param params: dict with server details
        :param cleanup: Boolean value, true to cleanup, false to setup
        """
        commands = []
        # check the existing iptables rules in remote or local machine
        iptable_check_cmd = "iptables -S"
        if params:
            server_ip = params.get("server_ip")
            server_user = params.get("server_user", "root")
            server_pwd = params.get("server_pwd")
            server_session = remote.wait_for_login('ssh', server_ip, '22',
                                                   server_user, server_pwd,
                                                   r"[\#\$]\s*$")
            cmd_output = server_session.cmd_status_output(iptable_check_cmd)
            if (cmd_output[0] == 0):
                exist_rules = cmd_output[1].strip().split('\n')
            else:
                server_session.close()
                raise exceptions.TestError("iptables fails for command "
                                           "remotely %s" % iptable_check_cmd)
        else:
            try:
                cmd_output = process.run(iptable_check_cmd,
                                         shell=True).stdout_text
                exist_rules = cmd_output.strip().split('\n')
            except process.CmdError as info:
                raise exceptions.TestError("iptables fails for command "
                                           "locally %s" % iptable_check_cmd)
        # check rules whether it is really needed to be added or cleaned
        for rule in rules:
            flag = False
            for exist_rule in exist_rules:
                if rule in exist_rule:
                    logging.debug("Rule: %s exist in iptables", rule)
                    flag = True
                    if cleanup:
                        logging.debug("cleaning rule: %s", rule)
                        commands.append("iptables -D %s" % rule)
            if not flag and not cleanup:
                logging.debug("Adding rule: %s", rule)
                commands.append("iptables -I %s" % rule)
        # Once rules are filtered, then it is executed in remote or local
        # machine
        for command in commands:
            if params:
                cmd_output = server_session.cmd_status_output(command)
                if (cmd_output[0] != 0):
                    server_session.close()
                    raise exceptions.TestError("iptables command failed "
                                               "remotely %s" % command)
                else:
                    logging.debug("iptable command success %s", command)
            else:
                try:
                    cmd_output = process.run(command, shell=True).stdout_text
                    logging.debug("iptable command success %s", command)
                except process.CmdError as info:
                    raise exceptions.TestError("iptables fails for command "
                                               "locally %s" % command)
        # cleanup server session
        if params:
            server_session.close()


class Firewall_cmd(object):
    """
    class handles firewall-cmd methods
    """
    def __init__(self, session=None):
        """
        initialises the firewall cmd objects

        :param session: ShellSession Object of guest/remote host
        """
        self.session = session
        self.firewall_cmd = "firewall-cmd"
        self.func = process.getstatusoutput
        if self.session:
            self.func = self.session.cmd_status_output

    def command(self, cmd, **dargs):
        """
        Wrapper method to execute firewall-cmd with different options

        :param cmd: firewall-cmd command options
        :param dargs: Additional arguments for the command
        """
        self.cmd = "%s %s" % (self.firewall_cmd, cmd)
        if dargs.get("permanent", False):
            self.cmd += " --permanent"
        # default zone to be public
        zone = dargs.get("zone", "public")
        if zone:
            self.cmd += " --zone=%s" % zone
        self.status, self.output = self.func(self.cmd)
        if self.status != 0:
            logging.error("Failed to execute %s: %s", self.cmd, self.output)
        # Reload the configuration to make effect at once
        if dargs.get("firewalld_reload", True):
            self.reload()

    def lists(self, key='all', **dargs):
        """
        Method to list existing services/ports etc.,

        :param key: key to be listed, eg: all, services etc.,
        :param dargs: Additional arguments for the command

        :return: output of the --list-*
        """
        cmd = "--list-%s" % key
        dargs['firewalld_reload'] = False
        self.command(cmd, **dargs)
        return self.output

    def get(self, key='zones', is_direct=False, **dargs):
        """
        Method to get existing zones/services etc.,

        :param key: key to be get from firewall-cmd
        :param is_direct: True to get with direct option
        :param dargs: Additional arguments for the command

        :return: output of the --get-*
        """
        cmd = "--get-%s" % key
        if is_direct:
            cmd = "--direct " + cmd
        dargs['firewalld_reload'] = False
        self.command(cmd, **dargs)
        return self.output

    def add_service(self, service_name, **dargs):
        """
        Method to add services to be permitted by firewall

        :param service_name: service name to be added
        :param dargs: Additional arguments for the command

        :return: True on success, False on failure.
        """
        cmd = "--add-service=%s" % service_name
        self.command(cmd, **dargs)
        return self.status == 0

    def remove_service(self, service_name, **dargs):
        """
        Method to remove services from permitted by firewall

        :param service_name: service name to be added
        :param dargs: Additional arguments for the command

        :return: True on success, False on failure.
        """
        cmd = "--remove-service=%s" % service_name
        self.command(cmd, **dargs)
        return self.status == 0

    def add_port(self, port, protocol, **dargs):
        """
        Method to add port to be permitted by firewall

        :param port: port number to be added
        :param protocol: protocol respective to the port to be added
        :param dargs: Additional arguments for the command

        :return: True on success, False on failure.
        """
        cmd = "--add-port=%s/%s" % (port, protocol)
        self.command(cmd, **dargs)
        return self.status == 0

    def remove_port(self, port, protocol, **dargs):
        """
        Method to remove port from permitted by firewall

        :param port: port number to be added
        :param protocol: protocol respective to the port to be added
        :param dargs: Additional arguments for the command

        :return: True on success, False on failure.
        """
        cmd = "--remove-port=%s/%s" % (port, protocol)
        self.command(cmd, **dargs)
        return self.status == 0

    def reload(self, complete=False):
        """
        Method to reload/complete reload

        :return: True on success, False on failure.
        """
        cmd = "firewall-cmd --reload"
        if complete:
            cmd = "firewall-cmd --complete-reload"
        self.status, self.output = self.func(cmd)

        return self.status == 0

    def add_direct_rule(self, rule, **dargs):
        """
        Method to add direct rule by firewall-cmd

        :param rule: Rule to be added
        :param dargs: Additional arguments for the command

        :return: True on success, False on failure.
        """
        dargs["zone"] = None
        dargs["firewalld_reload"] = False
        cmd = "--direct --add-rule %s" % (rule)
        self.command(cmd, **dargs)
        return self.status == 0

    def remove_direct_rule(self, rule, **dargs):
        """
        Method to remove direct rule by firewall-cmd

        :param rule: Rule to be removed
        :param dargs: Additional arguments for the command

        :return: True on success, False on failure.
        """
        dargs["zone"] = None
        dargs["firewalld_reload"] = False
        cmd = "--direct --remove-rule %s" % (rule)
        self.command(cmd, **dargs)
        return self.status == 0



"""
Classes to support XML for serial devices

http://libvirt.org/formatdomain.html#elementCharSerial
"""

from virttest.libvirt_xml import base, accessors, xcepts
from virttest.libvirt_xml.devices.character import CharacterBase


class Serial(CharacterBase):

    __slots__ = ('protocol_type', 'target_port', 'target_type',
                 'target_model', 'sources', 'alias', 'address')

    def __init__(self, type_name='pty', virsh_instance=base.virsh):
        # Additional attribute for protocol type (raw, telnet, telnets, tls)
        accessors.XMLAttribute('protocol_type', self, parent_xpath='/',
                               tag_name='protocol', attribute='type')
        accessors.XMLAttribute('target_port', self, parent_xpath='/',
                               tag_name='target', attribute='port')
        accessors.XMLAttribute('target_type', self, parent_xpath='/',
                               tag_name='target', attribute='type')
        accessors.XMLAttribute('target_model', self, parent_xpath='/target',
                               tag_name='model', attribute='name')
        accessors.XMLElementList('sources', self, parent_xpath='/',
                                 marshal_from=self.marshal_from_sources,
                                 marshal_to=self.marshal_to_sources)
        accessors.XMLElementDict('alias', self, parent_xpath='/',
                                 tag_name='alias')
        accessors.XMLElementDict('address', self, parent_xpath='/',
                                 tag_name='address')
        super(Serial, self).__init__(device_tag='serial', type_name=type_name,
                                     virsh_instance=virsh_instance)

    @staticmethod
    def marshal_from_sources(item, index, libvirtxml):
        """
        Convert a dict to serial source attributes.
        """
        del index
        del libvirtxml
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of source "
                                         "attributes, not a %s"
                                         % str(item))
        return ('source', dict(item))

    @staticmethod
    def marshal_to_sources(tag, attr_dict, index, libvirtxml):
        """
        Convert a source tag and attributes to a dict.
        """
        del index
        del libvirtxml
        if tag != 'source':
            return None
        return dict(attr_dict)



#!/usr/bin/python
import os


class check_error(Exception):
    pass


def check_cpu_flag():
    with open('/proc/cpuinfo') as info_f:
        cpuinfo = info_f.read()
    flags = os.environ['KVM_TEST_required_cpu_flags']
    for i in flags.split():
        if i not in cpuinfo:
            err_msg = "Host CPU doestn't have flag(%s)" % i
            print(err_msg)
            raise check_error(err_msg)


if __name__ == "__main__":
    check_cpu_flag()



"""
A decorator utility functions to apply libvirtd functions.

Copyright: Red Hat Inc. 2020
"""
import logging
import os
import re
from virttest import data_dir

from avocado.utils import process
from avocado.utils import path
from avocado.utils import astring


try:
    path.find_command("libvirtd")
    LIBVIRTD = "libvirtd"
except path.CmdNotFoundError:
    try:
        path.find_command("virtqemud")
        LIBVIRTD = "virtqemud"
    except path.CmdNotFoundError:
        LIBVIRTD = None


def get_libvirtd_split_enable_bit():
    base_cfg_path = os.path.join(data_dir.get_shared_dir(), 'cfg', 'base.cfg')
    if os.path.isfile(base_cfg_path):
        with open(base_cfg_path, 'r') as base_file:
            for line in base_file:
                if 'enable_split_libvirtd_feature' in line and 'yes' in line and '#' not in line:
                    return True
    else:
        logging.info("CAN NOT find base.cfg file")
    return False


def get_libvirt_version_compare(major, minor, update, session=None):
    """
    Determine/use the current libvirt library version on the system
    and compare input major, minor, and update values against it.
    If the running version is greater than or equal to the input
    params version, then return True; otherwise, return False.

    This is designed to handle upstream version comparisons for
    test adjustments and/or comparisons as a result of upstream
    fixes or changes that could impact test results.

    :param major: Major version to compare against
    :param minor: Minor version to compare against
    :param update: Update value to compare against
    :param session: Shell session on remote host
    :return: True if running version is greater than or
                  equal to the input libvirt version
    """
    LIBVIRT_LIB_VERSION = 0

    func = process.system_output
    if session:
        func = session.cmd_output

    if LIBVIRTD is None:
        logging.warn("Can not find command to dertermin libvirt version")
        return False
    libvirt_ver_cmd = "%s -V" % LIBVIRTD
    logging.warn(libvirt_ver_cmd)
    try:
        regex = r'%s\s*.*[Ll]ibvirt.*\s*' % LIBVIRTD
        regex += r'(\d+)\.(\d+)\.(\d+)'
        lines = astring.to_text(func(libvirt_ver_cmd)).splitlines()
        logging.warn("libvirt version value by libvirtd or virtqemud command: %s" % lines)
        for line in lines:
            match = re.search(regex, line.strip())
            if match:
                LIBVIRT_LIB_VERSION = int(match.group(1)) * 1000000 + int(match.group(2)) * 1000 + int(match.group(3))
                break
    except (ValueError, TypeError, AttributeError):
        logging.warn("Error determining libvirt version")
        return False

    compare_version = major * 1000000 + minor * 1000 + update
    if LIBVIRT_LIB_VERSION >= compare_version:
        return True
    return False


LIBVIRTD_SPLIT_ENABLE_BIT = None
IS_LIBVIRTD_SPLIT_VERSION = None


def check_libvirt_version():
    global LIBVIRTD_SPLIT_ENABLE_BIT
    global IS_LIBVIRTD_SPLIT_VERSION
    if LIBVIRTD_SPLIT_ENABLE_BIT is None:
        LIBVIRTD_SPLIT_ENABLE_BIT = get_libvirtd_split_enable_bit()
    if IS_LIBVIRTD_SPLIT_VERSION is None:
        IS_LIBVIRTD_SPLIT_VERSION = get_libvirt_version_compare(5, 6, 0)


def libvirt_version_context_aware_libvirtd_legacy(fn):
    """
    A decorator that must be applied to functions that call if libvirt version <5.6.0.

    :param fn: function name.
    """
    def new_fn(*args, **kwargs):
        """
        Keep previous function as working before if libvirt version< 5.6.0, else do nothing

        :param args: function fixed args.
        :param kwargs: function varied args.
        """
        check_libvirt_version()
        if not IS_LIBVIRTD_SPLIT_VERSION or not LIBVIRTD_SPLIT_ENABLE_BIT:
            logging.warn("legacy start libvirtd daemon NORMALLY with function name: %s" % fn.__name__)
            return fn(*args, **kwargs)
        else:
            logging.warn("legacy start libvirtd daemon IGNORED with function name: %s" % fn.__name__)
            return None
    return new_fn


def libvirt_version_context_aware_libvirtd_split(fn):
    """
    A decorator that must be applied to functions that call if libvirt version >=5.6.0

    :param fn: function name.
    """
    def new_fn(*args, **kwargs):
        """
        Keep previous function as working before if libvirt version>= 5.6.0, else do nothing

        :param args: function fixed args.
        :param kwargs: function varied args.
        """
        check_libvirt_version()
        if IS_LIBVIRTD_SPLIT_VERSION and LIBVIRTD_SPLIT_ENABLE_BIT:
            logging.warn("Split start libvirtd daemon NORMALLY with function name: %s" % fn.__name__)
            return fn(*args, **kwargs)
        else:
            logging.warn("Split start libvirtd daemon IGNORED with function name: %s" % fn.__name__)
            return None
    return new_fn



"""
Parallel device support class(es)

http://libvirt.org/formatdomain.html#elementCharSerial
"""

from virttest.libvirt_xml import base
from virttest.libvirt_xml.devices.character import CharacterBase


class Parallel(CharacterBase):

    __slots__ = []

    def __init__(self, type_name='pty', virsh_instance=base.virsh):
        super(Parallel, self).__init__(device_tag='parallel',
                                       type_name=type_name,
                                       virsh_instance=virsh_instance)



from virttest import error_context

from avocado.utils import process


@error_context.context_aware
def get_image_filename(curl_protocol, curl_server, curl_path,
                       curl_user=None, curl_passwd=None):
    """
    Form the url: <protocol>://[<username>[:<password>]@]<host>/<path>

    :param curl_protocol: one of 'http', 'https', 'ftp', 'ftps'
    :param curl_server: Address of the remote server
    :param curl_path: Path on the remote server, including any query string
    :param curl_user: Username for authentication to the remote server
    :param curl_passwd: Password for authentication to the remote server
    :return: Remote image filename in url format
    """
    url = "{protocol}://{auth}{host}/{path}"

    protocols = ('http', 'https', 'ftp', 'ftps')
    if curl_protocol not in protocols:
        raise ValueError('curl_protocol should be in %s.' % str(protocols))

    auth = ''
    if curl_user:
        auth = '%s:%s@' % (curl_user,
                           curl_passwd) if curl_passwd else '%s@' % curl_user

    return url.format(protocol=curl_protocol, auth=auth,
                      host=curl_server, path=curl_path)


@error_context.context_aware
def file_exists(params, filename):
    """
    Check whether the image on curl storage exists.

    :param params: A dict containing image parameters.
    :param filename: The image filename
    :return: True if the image exists, else False
    """
    curl_cmd = 'curl -I -L -k {tmo} {filename}'
    t = '-m %s' % params['curl_timeout'] if params.get('curl_timeout') else ''
    o = process.run(curl_cmd.format(tmo=t, filename=filename),
                    ignore_status=True, verbose=True).stdout_text.strip()
    return 'Content-Length:' in o



"""
Support for the iommu device XML

http://libvirt.org/formatdomain.html#elementsDevices
"""

from virttest.libvirt_xml import accessors
from virttest.libvirt_xml.devices import base


class Iommu(base.UntypedDeviceBase):

    __slots__ = ('model', 'driver')

    def __init__(self, virsh_instance=base.base.virsh):
        accessors.XMLAttribute(property_name='model',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='iommu',
                               attribute='model')
        accessors.XMLElementDict(property_name='driver',
                                 libvirtxml=self,
                                 parent_xpath='/',
                                 tag_name='driver')
        super(Iommu, self).__init__(device_tag='iommu',
                                    virsh_instance=virsh_instance)



"""
This module implements classes that perform the installation of the
virtualization software on a host system.

These classes can be, and usually are, inherited by subclasses that implement
custom logic for each virtualization hypervisor/software.
"""

import os
import logging

from avocado.core import exceptions
from avocado.utils import process
from avocado.utils import path
from avocado.utils import linux_modules

from . import build_helper
from . import utils_misc
from . import yumrepo
from . import arch
from .staging import utils_koji


class NoModuleError(Exception):

    """
    Error raised when no suitable modules were found to load
    """
    pass


class VirtInstallException(Exception):

    """
    Base virtualization software components installation exception
    """
    pass


class VirtInstallFailed(VirtInstallException):

    """
    Installation of virtualization software components failed
    """
    pass


class VirtInstallNotInstalled(VirtInstallException):

    """
    Virtualization software components are not installed
    """
    pass


class BaseInstaller(object):

    """
    Base virtualization software installer

    This class holds all the skeleton features for installers and should be
    inherited from when creating a new installer.
    """

    def __init__(self, mode, name, test=None, params=None):
        """
        Instantiates a new base installer

        :param mode: installer mode, such as git_repo, local_src, etc
        :param name: installer short name, foo for git_repo_foo
        :param test: test
        :param params: params
        """
        self.mode = mode
        self.name = name
        self.params = params
        self.test = test
        self.param_key_prefix = '%s_%s' % (self.mode,
                                           self.name)

        # If a installer has a failure that can be worked around, save that
        self.minor_failure = False
        self.minor_failure_reason = ''

        if test and params:
            self.set_install_params(test, params)

    def _set_test_dirs(self, test, params=None):
        """
        Save common test directories paths as class attributes

        Test variables values are saved here again because it's not possible to
        pickle the test instance inside BaseInstaller due to limitations
        in the pickle protocol. And, in case this pickle thing needs more
        explanation, take a loot at the Env class inside build_helper.

        Besides that, we also ensure that workdir exists, by creating it if
        necessary.

        For reference:
           * bindir = tests/<test>
           * workdir = tests/<test>/src
           * resultsdir = results/<job>/<testname.tag>/results

        So, for KVM tests, it'd evaluate to:
           * bindir = tests/kvm/
           * workdir = tests/kvm/src
           * resultsdir = results/<job>/kvm.<other_variant_names>.build/results
        """
        self.test_bindir = test.bindir
        if params.get("preserve_srcdir") == "yes":
            self.test_workdir = os.path.join(test.bindir, "build")
            self.test_builddir = os.path.join(test.bindir, "bin")
        else:
            self.test_workdir = test.workdir
            self.test_builddir = test.builddir
        self.test_resultsdir = test.resultsdir

        #
        # test_builddir is guaranteed to exist, but test_workdir is not
        #
        if not os.path.isdir(test.workdir):
            os.makedirs(test.workdir)

    def _set_param_load_module(self):
        """
        Checks whether kernel modules should be loaded

        Default behavior is to load modules unless set to 'no'

        Configuration file parameter: load_modules
        Class attribute set: should_load_modules
        """
        load_modules = self.params.get('load_modules', 'no')
        if not load_modules or load_modules == 'yes':
            self.should_load_modules = True
        elif load_modules == 'no':
            self.should_load_modules = False

    def _set_param_module_list(self):
        """
        Sets the list of kernel modules to be loaded during installation

        Configuration file parameter: module_list
        Class attribute set: module_list
        """
        self.module_list = self.params.get('module_list', [])

    def _set_param_save_results(self):
        """
        Checks whether to save the result of the build on test.resultsdir

        Configuration file parameter: save_results
        Class attribute set: save_results
        """
        self.save_results = True
        save_results = self.params.get('save_results', 'no')
        if save_results == 'no':
            self.save_results = False

    def _set_param_install_debug_info(self):
        """
        Sets whether to enable debug information on installed software

        Configuration file parameter: install_debug_info
        Class attribute set: install_debug_info
        """
        self.install_debug_info = True
        install_debug_info = self.params.get('install_debug_info', 'no')
        if install_debug_info == 'no':
            self.install_debug_info = False

    def _set_param_cleanup(self):
        """
        Sets whether to enable debug information on installed software

        Configuration file parameter: installer_cleanup
        Class attribute set: cleanup
        """
        self.cleanup = True
        cleanup = self.params.get('installer_cleanup', 'yes')
        if cleanup == 'no':
            logging.debug("Setting installer cleanup attribute to False")
            self.cleanup = False

    def set_install_params(self, test=None, params=None):
        """
        Called by test to setup parameters from the configuration file
        """
        logging.info("calling set install params")
        if test is not None:
            self._set_test_dirs(test, params)

        if params is not None:
            self.params = params
            self._set_param_load_module()
            self._set_param_module_list()
            self._set_param_save_results()
            self._set_param_install_debug_info()
            self._set_param_cleanup()

    def _install_phase_cleanup(self):
        """
        Optional install phase for removing previous version of the software

        If a particular virtualization software installation mechanism
        needs to download files (it most probably does), override this
        method with custom functionality.

        This replaces methods such as KojiInstaller._get_packages()
        """
        pass

    def _install_phase_cleanup_verify(self):
        """
        Optional install phase for removing previous version of the software

        If a particular virtualization software installation mechanism
        needs to download files (it most probably does), override this
        method with custom functionality.

        This replaces methods such as KojiInstaller._get_packages()
        """
        pass

    def _install_phase_download(self):
        """
        Optional install phase for downloading software

        If a particular virtualization software installation mechanism
        needs to download files (it most probably does), override this
        method with custom functionality.

        This replaces methods such as KojiInstaller._get_packages()
        """
        pass

    def _install_phase_download_verify(self):
        """
        Optional install phase for checking downloaded software

        If you want to make sure the downloaded software is in good shape,
        override this method.

        Ideas for using this method:
          * check MD5SUM/SHA1SUM for tarball downloads
          * check RPM files, probably by signature (rpm -k)
          * git status and check if there's no locally modified files
        """
        pass

    def _install_phase_prepare(self):
        """
        Optional install phase for preparing software

        If a particular virtualization software installation mechanism
        needs to do something to the obtained software, such as extracting
        a tarball or applying patches, this should be done here.
        """
        pass

    def _install_phase_prepare_verify(self):
        """
        Optional install phase for checking software preparation

        Ideas for using this method:
          * git status and check if there are locally patched files
        """
        pass

    def _install_phase_build(self):
        """
        Optional install phase for building software

        If a particular virtualization software installation mechanism
        needs to compile source code, it should be done here.
        """
        pass

    def _install_phase_build_verify(self):
        """
        Optional install phase for checking software build

        Ideas for using this method:
           * running 'make test' or something similar to it
        """
        pass

    def _install_phase_install(self):
        """
        Optional install phase for actually installing software

        Ideas for using this method:
           * running 'make install' or something similar to it
           * running 'yum localinstall *.rpm'
        """
        pass

    def _install_phase_install_verify(self):
        """
        Optional install phase for checking the installed software

        This should verify the installed software is in a desirable state.
        Ideas for using this include:
           * checking if installed files exists (like os.path.exists())
           * checking packages are indeed installed (rpm -q <pkg>.rpm)
        """
        pass

    def _install_phase_init(self):
        """
        Optional install phase for initializing the installed software

        This should initialize the installed software. Ideas for using this:
           * loading kernel modules
           * running services: 'service <daemon> start'
           * linking software (whether built or downloaded) to a common path
        """
        pass

    def _install_phase_init_verify(self):
        """
        Optional install phase for checking that software is initialized

        This should verify that the installed software is running. Ideas for
        using this include:
            * checking service (daemon) status: 'service <daemon> status'
            * checking service (functionality) status: 'virsh capabilities'
        """
        pass

    def _install_phase_package(self):
        """
        """
        pass

    def _install_phase_package_verify(self):
        '''
        Optional install phase for checking that software is packaged.

        This should verify that the packages are actually created. Ideas for
        using this include:
            * checking /root/rpmbuild/RPMS'
        '''
        pass

    def write_version_keyval(self, test):
        try:
            version = self.get_version()
        except AttributeError:
            version = "Unknown"
        sw_version = {('software_version_%s' % self.name): version}
        logging.debug("Writing test keyval %s", sw_version)
        test.write_test_keyval(sw_version)

    def load_modules(self, module_list=None):
        """
        Load Linux Kernel modules the virtualization software may depend on

        If module_directory is not set, the list of modules will simply be
        loaded by the system stock modprobe tool, meaning that modules will be
        looked for in the system default module paths.

        :type module_list: builtin.list
        :param module_list: list of kernel modules names to load
        """
        if module_list is None:
            module_list = self.module_list

        if not module_list:
            raise NoModuleError("Module list empty")

        logging.info("Loading modules from default locations through "
                     "modprobe")
        for module in module_list:
            process.system("modprobe %s" % module)

    def unload_modules(self, module_list=None):
        """
        Unloads kernel modules

        By default, if no module list is explicitly provided, the list on
        params (coming from the configuration file) will be used.
        """
        if module_list is None:
            module_list = self.module_list
        logging.info("Unloading kernel modules: %s" % " ".join(module_list))
        for module in module_list:
            linux_modules.unload_module(module)

    def reload_modules(self):
        """
        Reload the kernel modules (unload, then load)
        """
        self.unload_modules()
        self.load_modules()

    def reload_modules_if_needed(self):
        if self.should_load_modules:
            self.reload_modules()

    def install(self, cleanup=True, download=True, prepare=True,
                build=True, install=True, package=False, init=True):
        """
        Performs the installation of the virtualization software

        This is the main entry point of this class, and should  either
        be reimplemented completely, or simply implement one or many of the
        install  phases.
        """
        if (cleanup and self.cleanup):
            self._install_phase_cleanup()
            self._install_phase_cleanup_verify()

        if download:
            self._install_phase_download()
            self._install_phase_download_verify()

        if prepare:
            self._install_phase_prepare()
            self._install_phase_prepare_verify()

        if build:
            self._install_phase_build()
            self._install_phase_build_verify()

        if package:
            self._install_phase_package()
            self._install_phase_package_verify()

        if install:
            self._install_phase_install()
            self._install_phase_install_verify()

        if init:
            self._install_phase_init()
            self._install_phase_init_verify()

        self.reload_modules_if_needed()
        if self.save_results:
            utils_misc.archive_as_tarball(self.test_workdir,
                                          self.test_resultsdir)

    def uninstall(self):
        """
        Performs the uninstallations of the virtualization software

        Note: This replaces old qemu_installer._clean_previous_install()
        """
        raise NotImplementedError


class NoopInstaller(BaseInstaller):

    """
    Dummy installer that does nothing, useful when software is pre-installed
    """

    def __init__(self, mode, name, test=None, params=None):
        """
        If no previous install test ran, try to figure out modules to load.

        :param mode: Install mode (yum, git, etc).
        :param name: Installer name.
        :param test: Virt test object.
        :param params: Dict with test params.
        """
        if params['vm_type'] == 'qemu':
            params['module_list'] = arch.get_kvm_module_list()
        super(NoopInstaller, self).__init__(mode, name, test, params)

    def install(self):
        logging.info("Assuming virtualization software to be already "
                     "installed. Doing nothing")


class YumInstaller(BaseInstaller):

    """
    Installs virtualization software using YUM

    Notice: this class implements a change of behaviour if compared to
    qemu_installer.YumInstaller.set_install_params(). There's no longer
    a default package list, as each virtualization technology will have
    a completely different default. This should now be kept at the
    configuration file only.

    For now this class implements support for installing from the configured
    yum repos only. If the use case of installing from local RPM packages
    arises, we'll implement that.
    """

    def set_install_params(self, test, params):
        super(YumInstaller, self).set_install_params(test, params)
        path.find_command("rpm")
        path.find_command("yum")
        if self.install_debug_info:
            path.find_command("debuginfo-install")
        self.yum_pkgs = eval(params.get("%s_pkgs" % self.param_key_prefix,
                                        "[]"))

    def get_version(self):
        return " ".join(self.yum_pkgs)

    def _install_phase_cleanup(self):
        packages_to_remove = " ".join(self.yum_pkgs)
        process.system("yum remove -y %s" % packages_to_remove)

    def _install_phase_package(self):
        if self.build_helper is not None:
            self.build_helper.package()

    def _install_phase_install(self):
        if self.yum_pkgs:
            os.chdir(self.test_workdir)
            process.system("yum --nogpgcheck -y install %s" %
                           " ".join(self.yum_pkgs))
        if self.install_debug_info:
            process.system("debuginfo-install --enablerepo='*debuginfo' -y %s" %
                           " ".join(self.yum_pkgs))


class KojiInstaller(BaseInstaller):

    """
    Handles virtualization software installation via koji/brew

    It uses YUM to install and remove packages.

    Change notice: this is not a subclass of YumInstaller anymore. The
    parameters this class uses are different (koji_tag, koji_pgks) and
    the install process runs YUM.
    """

    def set_install_params(self, test, params):
        super(KojiInstaller, self).set_install_params(test, params)
        path.find_command("rpm")
        path.find_command("yum")

        self.tag = params.get("%s_tag" % self.param_key_prefix, None)
        self.koji_cmd = params.get("%s_cmd" % self.param_key_prefix, None)
        if self.tag is not None:
            utils_koji.set_default_koji_tag(self.tag)
        self.koji_pkgs = params.get("%s_pkgs" % self.param_key_prefix,
                                    "").split()
        self.koji_scratch_pkgs = params.get("%s_scratch_pkgs" %
                                            self.param_key_prefix,
                                            "").split()
        self.koji_yumrepo_baseurl = params.get("%s_yumrepo_baseurl" %
                                               self.param_key_prefix, None)
        if self.install_debug_info:
            self._expand_koji_pkgs_with_debuginfo()

    def get_version(self):
        return " ".join(self._get_rpm_file_names())

    def _expand_koji_pkgs_with_debuginfo(self):
        """
        Include debuginfo RPMs on koji_pkgs

        :return: None
        """
        logging.debug("Koji package list to be updated with debuginfo pkgs")

        koji_pkgs_with_debug = []
        for pkg_text in self.koji_pkgs:
            pkg = utils_koji.KojiPkgSpec(pkg_text)
            debuginfo_pkg_name = '%s-debuginfo' % pkg.package
            # if no subpackages are set, then all packages will be installed
            # so there's no need to manually include debuginfo packages
            if pkg.subpackages:
                # make sure we do not include the debuginfo package if
                # already specified in the list of subpackages
                if debuginfo_pkg_name not in pkg.subpackages:
                    pkg.subpackages.append(debuginfo_pkg_name)

            pkg_with_debug_text = pkg.to_text()
            logging.debug("KojiPkgSpec with debuginfo package added: %s",
                          pkg_with_debug_text)
            koji_pkgs_with_debug.append(pkg_with_debug_text)

        # swap current koji_pkgs with on that includes debuginfo pkgs
        self.koji_pkgs = koji_pkgs_with_debug

    def _get_rpm_names(self):
        all_rpm_names = []
        koji_client = utils_koji.KojiClient(cmd=self.koji_cmd)
        for pkg_text in self.koji_pkgs:
            pkg = utils_koji.KojiPkgSpec(pkg_text)
            rpm_names = koji_client.get_pkg_rpm_names(pkg)
            all_rpm_names += rpm_names
        for scratch_pkg_text in self.koji_scratch_pkgs:
            pkg = utils_koji.KojiScratchPkgSpec(scratch_pkg_text)
            rpm_urls = koji_client.get_scratch_pkg_urls(pkg)
            file_names = list(map(os.path.basename, rpm_urls))
            for f in file_names:
                r = utils_koji.RPMFileNameInfo(f)
                all_rpm_names.append(r.get_nvr_info()['name'])
        return all_rpm_names

    def _get_rpm_file_names(self):
        all_rpm_file_names = []
        koji_client = utils_koji.KojiClient(cmd=self.koji_cmd)
        for pkg_text in self.koji_pkgs:
            pkg = utils_koji.KojiPkgSpec(pkg_text)
            rpm_file_names = koji_client.get_pkg_rpm_file_names(pkg)
            all_rpm_file_names += rpm_file_names
        for scratch_pkg_text in self.koji_scratch_pkgs:
            pkg = utils_koji.KojiScratchPkgSpec(scratch_pkg_text)
            rpm_urls = koji_client.get_scratch_pkg_urls(pkg)
            file_names = list(map(os.path.basename, rpm_urls))
            all_rpm_file_names += file_names
        return all_rpm_file_names

    def _install_phase_cleanup(self):
        removable_packages = " ".join(self._get_rpm_names())
        process.system("yum -y remove %s" % removable_packages)

    def _install_phase_download(self):
        koji_client = utils_koji.KojiClient(cmd=self.koji_cmd)
        for pkg_text in self.koji_pkgs:
            pkg = utils_koji.KojiPkgSpec(pkg_text)
            if pkg.is_valid():
                koji_client.get_pkgs(pkg, dst_dir=self.test_workdir)
            else:
                logging.error('Package specification (%s) is invalid: %s' %
                              (pkg, pkg.describe_invalid()))
        for pkg_text in self.koji_scratch_pkgs:
            pkg = utils_koji.KojiScratchPkgSpec(pkg_text)
            koji_client.get_scratch_pkgs(pkg, dst_dir=self.test_workdir)

    def _install_phase_install(self):
        if self.koji_yumrepo_baseurl is not None:
            repo = yumrepo.YumRepo(self.param_key_prefix,
                                   self.koji_yumrepo_baseurl)
            logging.debug('Enabling YUM Repo "%s" at "%s"',
                          self.param_key_prefix, self.koji_yumrepo_baseurl)
            repo.save()

        os.chdir(self.test_workdir)
        rpm_file_names = " ".join(self._get_rpm_file_names())
        process.system("yum --nogpgcheck -y install %s" % rpm_file_names)

        if self.koji_yumrepo_baseurl is not None:
            logging.debug('Disabling YUM Repo "%s" at "%s"',
                          self.param_key_prefix, self.koji_yumrepo_baseurl)
            repo.remove()


class BaseLocalSourceInstaller(BaseInstaller):

    def set_install_params(self, test, params):
        super(BaseLocalSourceInstaller, self).set_install_params(test, params)
        self._set_install_prefix()
        self._set_source_destination()

        #
        # There are really no choices for patch helpers
        #
        self.patch_helper = build_helper.PatchParamHelper(
            self.params,
            self.param_key_prefix,
            self.source_destination)

        #
        # These helpers should be set by child classes
        #
        self.content_helper = None
        self.build_helper = None

    def _set_install_prefix(self):
        """
        Prefix for installation of application built from source

        When installing virtualization software from *source*, this is where
        the resulting binaries will be installed. Usually this is the value
        passed to the configure script, ie: ./configure --prefix=<value>
        """
        prefix = os.path.join(self.test_builddir, 'install_root')
        self.install_prefix = os.path.abspath(prefix)

    def _set_source_destination(self):
        """
        Sets the source code destination directory path
        """
        self.source_destination = os.path.join(self.test_workdir,
                                               self.name)

    def _set_build_helper(self):
        """
        Sets the build helper, default is 'gnu_autotools'
        """
        build_helper_name = self.params.get('%s_build_helper' %
                                            self.param_key_prefix,
                                            'gnu_autotools')
        if build_helper_name == 'gnu_autotools':
            self.build_helper = build_helper.GnuSourceBuildParamHelper(
                self.params, self.param_key_prefix,
                self.source_destination, self.install_prefix)
        elif build_helper_name == 'linux_kernel':
            self.build_helper = build_helper.LinuxKernelBuildHelper(
                self.params, self.param_key_prefix,
                self.source_destination)

    def _install_phase_prepare(self):
        if self.patch_helper is not None:
            self.patch_helper.execute()

    def _install_phase_download(self):
        if self.content_helper is not None:
            self.content_helper.execute()

    def _install_phase_build(self):
        if self.build_helper is not None:
            if (isinstance(self.build_helper,
                           build_helper.GnuSourceBuildHelper) or
                isinstance(self.build_helper,
                           build_helper.LinuxKernelBuildHelper)):
                try:
                    self.build_helper.execute()
                except build_helper.SourceBuildParallelFailed:
                    # Flag minor the failure
                    self.minor_failure = True
                    self.minor_failure_reason = "Failed to do parallel build"

                except build_helper.SourceBuildFailed:
                    # Failed the current test
                    raise exceptions.TestError(
                        "Failed to build %s" % self.name)
            else:
                self.build_helper.execute()

    def _install_phase_install(self):
        if self.build_helper is not None:
            self.build_helper.install()


class LocalSourceDirInstaller(BaseLocalSourceInstaller):

    """
    Handles software installation by building/installing from a source dir
    """

    def set_install_params(self, test, params):
        super(LocalSourceDirInstaller, self).set_install_params(test, params)

        self.content_helper = build_helper.LocalSourceDirParamHelper(
            params,
            self.name,
            self.source_destination)

        self._set_build_helper()


class LocalSourceTarInstaller(BaseLocalSourceInstaller):

    """
    Handles software installation by building/installing from a tarball
    """

    def set_install_params(self, test, params):
        super(LocalSourceTarInstaller, self).set_install_params(test, params)

        self.content_helper = build_helper.LocalTarParamHelper(
            params,
            self.name,
            self.source_destination)

        self._set_build_helper()


class RemoteSourceTarInstaller(BaseLocalSourceInstaller):

    """
    Handles software installation by building/installing from a remote tarball
    """

    def set_install_params(self, test, params):
        super(RemoteSourceTarInstaller, self).set_install_params(test, params)

        self.content_helper = build_helper.RemoteTarParamHelper(
            params,
            self.name,
            self.source_destination)

        self._set_build_helper()


class GitRepoInstaller(BaseLocalSourceInstaller):

    def set_install_params(self, test, params):
        super(GitRepoInstaller, self).set_install_params(test, params)

        self.content_helper = build_helper.GitRepoParamHelper(
            params,
            self.name,
            self.source_destination)

        self._set_build_helper()

    def get_version(self):
        uri = self.content_helper.uri
        branch = self.content_helper.branch
        commit = self.content_helper.get_top_commit()
        return "%s:%s:%s" % (uri, branch, commit)


class FailedInstaller:

    """
    Class used to be returned instead of the installer if a installation fails

    Useful to make sure no installer object is used if virt installation fails
    """

    def __init__(self, msg="Virtualization software install failed"):
        self._msg = msg

    def load_modules(self):
        """
        Will refuse to load the kerkel modules as install failed
        """
        raise VirtInstallFailed("Kernel modules not available. reason: %s" %
                                self._msg)



#!/usr/bin/env python

'''
Created on Dec 6, 2013

:author: jzupka, astepano
:contact: Andrei Stepanov <astepano@redhat.com>
'''

import os
import logging
import select
import time
import base64
import importlib

try:
    from cStringIO import StringIO
except ImportError:
    from six import StringIO
try:
    import pickle as cPickle
except ImportError:
    import cPickle

if __package__ is None:  # import when remote_runner.py script run directly
    remote_interface = importlib.import_module('remote_interface')
else:
    from virttest.remote_commander import remote_interface


class IOWrapper(object):

    """
    Class encaptulates io opearation to be more consist in different
    implementations. (stdio, sockets, etc..)
    """

    def __init__(self, obj):
        """
        :param obj: IO obj for example file decriptor.
        """
        self._obj = obj

    def close(self):
        raise NotImplementedError()

    def read(self, max_len, timeout=None):
        """
        Read function should be reinmplemented as blocking reading from data
        source when timeout is None and nonblocking for timeout is not None.
        Implementation example StdIWrapper.

        :params max_len: Max len of readed data.
        :type max_len: int
        :param timeout: Timeout of reading operation.
        :type timeout: float
        :return: Readed data.
        """
        raise NotImplementedError()

    def write(self, data):
        """
        Write funciton should be implemented for object uded for writing.

        :param data: Data to write.
        :type data: str.
        """
        raise NotImplementedError()

    def fileno(self):
        """
        Function should return file descriptor number. If object should be used
        for standard io operation.

        :return: File number.
        """
        raise NotImplementedError()

    def _wait_for_data(self, max_len, timeout):
        """
        Wait for data for time == timeout.

        :params max_len: Max len of readed data.
        :type max_len: int
        :param timeout: Timeout of reading operation.
        :type timeout: float
        :return: Readed data.
        """
        r, _, _ = select.select([self.fileno()], [], [], timeout)
        if r:
            return self.read(max_len, None)
        return None


class DataWrapper(object):

    """
    Basic implementation of IOWrapper for stdio.
    """

    def decode(self, data):
        """
        Decodes the data which was read.

        :return: decoded data.
        """
        return data

    def encode(self, data):
        """
        Encode data.

        :return: encoded data.
        """
        return data


class DataWrapperBase64(DataWrapper):

    """
    Basic implementation of IOWrapper for stdio.
    """

    def decode(self, data):
        return base64.b64decode(data)

    def encode(self, data):
        return base64.b64encode(data)


class StdIOWrapper(IOWrapper, DataWrapper):

    """
    Basic implementation of IOWrapper for stdio.
    """

    def close(self):
        os.close(self._obj)

    def fileno(self):
        return self._obj


class StdIOWrapperIn(StdIOWrapper):

    """
    Basic implementation of IOWrapper for stdin
    """

    def read(self, max_len, timeout=None):
        if timeout is not None:
            return self._wait_for_data(max_len, timeout)
        else:
            return os.read(self._obj, max_len)


class StdIOWrapperOut(StdIOWrapper):

    """
    Basic implementation of IOWrapper for stdout
    """

    def write(self, data):
        os.write(self._obj, data)


class StdIOWrapperInBase64(StdIOWrapperIn, DataWrapperBase64):

    """
    Basic implementation of IOWrapper for stdin
    """


class StdIOWrapperOutBase64(StdIOWrapperOut, DataWrapperBase64):

    """
    Basic implementation of IOWrapper for stdout
    """


class MessengerError(Exception):

    def __init__(self, msg):
        super(MessengerError, self).__init__(msg)
        self.msg = msg

    def __str__(self):
        return "Messenger ERROR %s" % (self.msg)


def _map_path(mod_name, kls_name):
    if mod_name.endswith('remote_interface'):  # catch all old module names
        mod = remote_interface
        return getattr(mod, kls_name)
    else:
        mod = __import__(mod_name)
        return getattr(mod, kls_name)


class Messenger(object):

    """
    Class could be used for communication between two python process connected
    by communication canal wrapped by IOWrapper class. Pickling is used
    for communication and thus it is possible to communicate every picleable
    object.
    """

    def __init__(self, stdin, stdout):
        """
        :params stdin: Object for read data from communication interface.
        :type stdin: IOWrapper
        :params stdout: Object for write data to communication interface.
        :type stdout: IOWrapper
        """
        self.stdin = stdin
        self.stdout = stdout

        # Unfortunately only static length of data length is supported.
        self.enc_len_length = len(stdout.encode("0" * 10))

    def close(self):
        self.stdin.close()
        self.stdout.close()

    def format_msg(self, data):
        """
        Format message where first 10 char is length of message and rest is
        piclked message.
        """
        pdata = cPickle.dumps(data, cPickle.HIGHEST_PROTOCOL)
        pdata = self.stdout.encode(pdata)
        len_enc = self.stdout.encode("%10d" % len(pdata))
        return "%s%s" % (len_enc, pdata)

    def flush_stdin(self):
        """
        Flush all input data from communication interface.
        """
        const = 16384
        r, _, _ = select.select([self.stdin.fileno()], [], [], 1)
        while r:
            if len(self.stdin.read(const)) < const:
                break
            r, _, _ = select.select([self.stdin.fileno()], [], [], 1)

    def write_msg(self, data):
        """
        Write formated message to communication interface.
        """
        self.stdout.write(self.format_msg(data))

    def _read_until_len(self, timeout=None):
        """
        Deal with terminal interfaces... Read input until gets string
        contains " " and digits len(string) == 10

        :param timeout: timeout of reading.
        """
        data = ""

        endtime = None
        if timeout is not None:
            endtime = time.time() + timeout

        while (len(data) < self.enc_len_length and
               (endtime is None or time.time() < endtime)):
            d = self.stdin.read(1, timeout)
            if d is None:
                return None
            if len(d) == 0:
                return d
            data += d
        if len(data) < self.enc_len_length:
            return None

        return self.stdout.decode(data)

    def read_msg(self, timeout=None):
        """
        Read data from com interface.

        :param timeout: timeout for reading data.
        :type timeout: float
        :return: (True, data) when reading is successful.
                 (False, None) when other side is closed.
                 (None, None) when reading is timeouted.
        """
        data = self._read_until_len(timeout)
        if data is None:
            return (None, None)
        if len(data) == 0:
            return (False, None)
        rdata = None
        try:
            cmd_len = int(data)
            rdata = ""
            rdata_len = 0
            while (rdata_len < cmd_len):
                rdata += self.stdin.read(cmd_len - rdata_len)
                rdata_len = len(rdata)
            rdataIO = StringIO(self.stdin.decode(rdata))
            unp = cPickle.Unpickler(rdataIO)
            if cPickle.__name__ == 'pickle':
                unp.find_class = _map_path
            else:
                unp.find_global = _map_path
            data = unp.load()
        except Exception as e:
            logging.error("ERROR data:%s rdata:%s" % (data, rdata))
            try:
                self.write_msg(remote_interface.MessengerError("Communication "
                                                               "failed.%s" % (e)))
            except OSError:
                pass
            self.flush_stdin()
            raise
        # Debugging commands.
        # if (isinstance(data, remote_interface.BaseCmd)):
        #    print data.func
        return (True, data)



#!/usr/bin/python
"""
Postprocessing module for IOzone. It is capable to pick results from an
IOzone run, calculate the geometric mean for all throughput results for
a given file size or record size, and then generate a series of 2D and 3D
graphs. The graph generation functionality depends on gnuplot, and if it
is not present, functionality degrates gracefully.

:copyright: Red Hat 2010
"""

from __future__ import division
import os
import sys
import optparse
import logging
import math
import time

from avocado.utils import process
from avocado.utils import path

from . import utils_misc

_LABELS = ['file_size', 'record_size', 'write', 'rewrite', 'read', 'reread',
           'randread', 'randwrite', 'bkwdread', 'recordrewrite', 'strideread',
           'fwrite', 'frewrite', 'fread', 'freread']


def geometric_mean(values):
    """
    Evaluates the geometric mean for a list of numeric values.

    :param values: List with values.
    :return: Single value representing the geometric mean for the list values.
    :see: `Geometric mean definition <http://en.wikipedia.org/wiki/Geometric_mean>`__
    """
    try:
        values = [int(value) for value in values]
    except ValueError:
        return None
    n = len(values)
    if n == 0:
        return None
    return math.exp(sum([math.log(x) for x in values]) / n)


def compare_matrices(matrix1, matrix2, treshold=0.05):
    """
    Compare 2 matrices nxm and return a matrix nxm with comparison data

    :param matrix1: Reference Matrix with numeric data
    :param matrix2: Matrix that will be compared
    :param treshold: Any difference bigger than this percent treshold will be
            reported.
    """
    improvements = 0
    regressions = 0
    same = 0

    new_matrix = []
    for line1, line2 in list(zip(matrix1, matrix2)):
        new_line = []
        for element1, element2 in list(zip(line1, line2)):
            ratio = float(element2) / float(element1)
            if ratio < (1 - treshold):
                regressions += 1
                new_line.append((100 * ratio - 1) - 100)
            elif ratio > (1 + treshold):
                improvements += 1
                new_line.append("+" + str((100 * ratio - 1) - 100))
            else:
                same + 1
                if line1.index(element1) == 0:
                    new_line.append(element1)
                else:
                    new_line.append(".")
        new_matrix.append(new_line)

    total = improvements + regressions + same

    return (new_matrix, improvements, regressions, total)


class IOzoneAnalyzer(object):

    """
    Analyze an unprocessed IOzone file, and generate the following types of
    report:

    * Summary of throughput for all file and record sizes combined
    * Summary of throughput for all file sizes
    * Summary of throughput for all record sizes

    If more than one file is provided to the analyzer object, a comparison
    between the two runs is made, searching for regressions in performance.
    """

    def __init__(self, list_files, output_dir):
        self.list_files = list_files
        if not os.path.isdir(output_dir):
            os.makedirs(output_dir)
        self.output_dir = output_dir
        logging.info("Results will be stored in %s", output_dir)

    def average_performance(self, results, size=None):
        """
        Flattens a list containing performance results.

        :param results: List of n lists containing data from performance runs.
        :param size: Numerical value of a size (say, file_size) that was used
                to filter the original results list.
        :return: List with 1 list containing average data from the performance
                run.
        """
        average_line = []
        if size is not None:
            average_line.append(size)
        for i in range(2, 15):
            average = geometric_mean([line[i] for line in results]) / 1024.0
            average = int(average)
            average_line.append(average)
        return average_line

    def process_results(self, results, label=None):
        """
        Process a list of IOzone results according to label.

        :param label: IOzone column label that we'll use to filter and compute
                    geometric mean results, in practical term either 'file_size'
                    or 'record_size'.
        :param result: A list of n x m columns with original iozone results.
        :return: A list of n-? x (m-1) columns with geometric averages for
                values of each label (ex, average for all file_sizes).
        """
        performance = []
        if label is not None:
            index = _LABELS.index(label)
            sizes = utils_misc.unique([line[index] for line in results])
            sizes.sort()
            for size in sizes:
                r_results = [line for line in results if line[index] == size]
                performance.append(self.average_performance(r_results, size))
        else:
            performance.append(self.average_performance(results))

        return performance

    def parse_file(self, fileobj):
        """
        Parse an IOzone results file.

        :param file: File object that will be parsed.
        :return: Matrix containing IOzone results extracted from the file.
        """
        lines = []
        for line in fileobj.readlines():
            fields = line.split()
            if len(fields) != 15:
                continue
            try:
                lines.append([int(i) for i in fields])
            except ValueError:
                continue
        return lines

    def report(self, overall_results, record_size_results, file_size_results):
        """
        Generates analysis data for IOZone run.

        Generates a report to both logs (where it goes with nice headers) and
        output files for further processing (graph generation).

        :param overall_results: 1x15 Matrix containing IOzone results for all
                file sizes
        :param record_size_results: nx15 Matrix containing IOzone results for
                each record size tested.
        :param file_size_results: nx15 Matrix containing file size results
                for each file size tested.
        """
        # Here we'll use the logging system to put the output of our analysis
        # to files
        logger = logging.getLogger()
        formatter = logging.Formatter("")

        logging.info("")
        logging.info(
            "TABLE:  SUMMARY of ALL FILE and RECORD SIZES                        Results in MB/sec")
        logging.info("")
        logging.info(
            "FILE & RECORD  INIT    RE              RE    RANDOM  RANDOM  BACKWD   RECRE  STRIDE    F       FRE     F       FRE")
        logging.info(
            "SIZES (KB)     WRITE   WRITE   READ    READ    READ   WRITE    READ   WRITE    READ    WRITE   WRITE   READ    READ")
        logging.info(
            "-------------------------------------------------------------------------------------------------------------------")
        for result_line in overall_results:
            logging.info(
                "ALL            %-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s" %
                tuple(result_line))
        logging.info("")

        logging.info("DRILLED DATA:")

        logging.info("")
        logging.info(
            "TABLE:  RECORD Size against all FILE Sizes                          Results in MB/sec")
        logging.info("")
        logging.info(
            "RECORD    INIT    RE              RE    RANDOM  RANDOM  BACKWD   RECRE  STRIDE    F       FRE     F       FRE ")
        logging.info(
            "SIZE (KB) WRITE   WRITE   READ    READ    READ   WRITE    READ   WRITE    READ    WRITE   WRITE   READ    READ")
        logging.info(
            "--------------------------------------------------------------------------------------------------------------")

        foutput_path = os.path.join(self.output_dir, '2d-datasource-file')
        if os.path.isfile(foutput_path):
            os.unlink(foutput_path)
        foutput = logging.FileHandler(foutput_path)
        foutput.setFormatter(formatter)
        logger.addHandler(foutput)
        for result_line in record_size_results:
            logging.info(
                "%-10s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s" %
                tuple(result_line))
        logger.removeHandler(foutput)

        logging.info("")

        logging.info("")
        logging.info(
            "TABLE:  FILE Size against all RECORD Sizes                          Results in MB/sec")
        logging.info("")
        logging.info(
            "RECORD    INIT    RE              RE    RANDOM  RANDOM  BACKWD   RECRE  STRIDE    F       FRE     F       FRE ")
        logging.info(
            "SIZE (KB) WRITE   WRITE   READ    READ    READ   WRITE    READ   WRITE    READ    WRITE   WRITE   READ    READ")
        logging.info(
            "--------------------------------------------------------------------------------------------------------------")

        routput_path = os.path.join(self.output_dir, '2d-datasource-record')
        if os.path.isfile(routput_path):
            os.unlink(routput_path)
        routput = logging.FileHandler(routput_path)
        routput.setFormatter(formatter)
        logger.addHandler(routput)
        for result_line in file_size_results:
            logging.info(
                "%-10s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s%-8s" %
                tuple(result_line))
        logger.removeHandler(routput)

        logging.info("")

    def report_comparison(self, record, file_size_results):
        """
        Generates comparison data for 2 IOZone runs.

        It compares 2 sets of nxm results and outputs a table with differences.
        If a difference higher or smaller than 5% is found, a warning is
        triggered.

        :param record: Tuple with 4 elements containing results for record size.
        :param file_size_results: Tuple with 4 elements containing results for file size.
        """
        (record_size, record_improvements, record_regressions,
         record_total) = record
        (file_size, file_improvements, file_regressions,
         file_total) = file_size_results
        logging.info("ANALYSIS of DRILLED DATA:")

        logging.info("")
        logging.info(
            "TABLE:  RECsize Difference between runs                            Results are % DIFF")
        logging.info("")
        logging.info(
            "RECORD    INIT    RE              RE    RANDOM  RANDOM  BACKWD   RECRE  STRIDE    F       FRE     F       FRE ")
        logging.info(
            "SIZE (KB) WRITE   WRITE   READ    READ    READ   WRITE    READ   WRITE    READ    WRITE   WRITE   READ    READ")
        logging.info(
            "--------------------------------------------------------------------------------------------------------------")
        for result_line in record_size:
            logging.info(
                "%-10s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s" %
                tuple(result_line))
        logging.info("REGRESSIONS: %d (%.2f%%)    Improvements: %d (%.2f%%)",
                     record_regressions,
                     (100 * record_regressions / float(record_total)),
                     record_improvements,
                     (100 * record_improvements / float(record_total)))
        logging.info("")

        logging.info("")
        logging.info(
            "TABLE:  FILEsize Difference between runs                           Results are % DIFF")
        logging.info("")
        logging.info(
            "RECORD    INIT    RE              RE    RANDOM  RANDOM  BACKWD   RECRE  STRIDE    F       FRE     F       FRE ")
        logging.info(
            "SIZE (KB) WRITE   WRITE   READ    READ    READ   WRITE    READ   WRITE    READ    WRITE   WRITE   READ    READ")
        logging.info(
            "--------------------------------------------------------------------------------------------------------------")
        for result_line in file_size:
            logging.info(
                "%-10s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s%-8.6s" %
                tuple(result_line))
        logging.info("REGRESSIONS: %d (%.2f%%)    Improvements: %d (%.2f%%)",
                     file_regressions,
                     (100 * file_regressions / float(file_total)),
                     file_improvements,
                     (100 * file_improvements / float(file_total)))
        logging.info("")

    def analyze(self):
        """
        Analyzes and eventually compares sets of IOzone data.
        """
        overall = []
        record_size = []
        file_size = []
        for file_path in self.list_files:
            fileobj = open(file_path, 'r')
            logging.info('FILE: %s', file_path)

            results = self.parse_file(fileobj)

            overall_results = self.process_results(results)
            record_size_results = self.process_results(results, 'record_size')
            file_size_results = self.process_results(results, 'file_size')
            self.report(
                overall_results, record_size_results, file_size_results)

            if len(self.list_files) == 2:
                overall.append(overall_results)
                record_size.append(record_size_results)
                file_size.append(file_size_results)

        if len(self.list_files) == 2:
            record_comparison = compare_matrices(*record_size)
            file_comparison = compare_matrices(*file_size)
            self.report_comparison(record_comparison, file_comparison)


class IOzonePlotter(object):

    """
    Plots graphs based on the results of an IOzone run.

    Plots graphs based on the results of an IOzone run. Uses gnuplot to
    generate the graphs.
    """

    def __init__(self, results_file, output_dir):
        self.active = True
        try:
            self.gnuplot = path.find_command("gnuplot")
        except path.CmdNotFoundError:
            logging.error("Command gnuplot not found, disabling graph "
                          "generation")
            self.active = False

        if not os.path.isdir(output_dir):
            os.makedirs(output_dir)
        self.output_dir = output_dir

        if not os.path.isfile(results_file):
            logging.error("Invalid file %s provided, disabling graph "
                          "generation", results_file)
            self.active = False
            self.results_file = None
        else:
            self.results_file = results_file
            self.generate_data_source()

    def generate_data_source(self):
        """
        Creates data file without headers for gnuplot consumption.
        """
        results_file = open(self.results_file, 'r')
        self.datasource = os.path.join(self.output_dir, '3d-datasource')
        datasource = open(self.datasource, 'w')
        for line in results_file.readlines():
            fields = line.split()
            if len(fields) != 15:
                continue
            try:
                for i in fields:
                    int(i)
                datasource.write(line)
            except ValueError:
                continue
        datasource.close()

    def plot_2d_graphs(self):
        """
        For each one of the throughput parameters, generate a set of gnuplot
        commands that will create a parametric surface with file size vs.
        record size vs. throughput.
        """
        datasource_2d = os.path.join(self.output_dir, '2d-datasource-file')
        for index, label in list(zip(list(range(2, 15)), _LABELS[2:])):
            commands_path = os.path.join(self.output_dir, '2d-%s.do' % label)
            commands = ""
            commands += "set title 'Iozone performance: %s'\n" % label
            commands += "set logscale x\n"
            commands += "set xlabel 'File size (KB)'\n"
            commands += "set ylabel 'Througput (MB/s)'\n"
            commands += "set terminal png small size 450 350\n"
            commands += "set output '%s'\n" % os.path.join(self.output_dir,
                                                           '2d-%s.png' % label)
            commands += ("plot '%s' using 1:%s title '%s' with lines \n" %
                         (datasource_2d, index, label))
            commands_file = open(commands_path, 'w')
            commands_file.write(commands)
            commands_file.close()
            try:
                process.system("%s %s" % (self.gnuplot, commands_path))
            except process.CmdError:
                logging.error("Problem plotting from commands file %s",
                              commands_path)

    def plot_3d_graphs(self):
        """
        For each one of the throughput parameters, generate a set of gnuplot
        commands that will create a parametric surface with file size vs.
        record size vs. throughput.
        """
        for index, label in list(zip(list(range(1, 14)), _LABELS[2:])):
            commands_path = os.path.join(self.output_dir, '%s.do' % label)
            commands = ""
            commands += "set title 'Iozone performance: %s'\n" % label
            commands += "set grid lt 2 lw 1\n"
            commands += "set surface\n"
            commands += "set parametric\n"
            commands += "set xtics\n"
            commands += "set ytics\n"
            commands += "set logscale x 2\n"
            commands += "set logscale y 2\n"
            commands += "set logscale z\n"
            commands += "set xrange [2.**5:2.**24]\n"
            commands += "set xlabel 'File size (KB)'\n"
            commands += "set ylabel 'Record size (KB)'\n"
            commands += "set zlabel 'Througput (KB/s)'\n"
            commands += "set style data lines\n"
            commands += "set dgrid3d 80,80, 3\n"
            commands += "set terminal png small size 900 700\n"
            commands += "set output '%s'\n" % os.path.join(self.output_dir,
                                                           '%s.png' % label)
            commands += ("splot '%s' using 1:2:%s title '%s'\n" %
                         (self.datasource, index, label))
            commands_file = open(commands_path, 'w')
            commands_file.write(commands)
            commands_file.close()
            try:
                process.system("%s %s" % (self.gnuplot, commands_path))
            except process.CmdError:
                logging.error("Problem plotting from commands file %s",
                              commands_path)

    def plot_all(self):
        """
        Plot all graphs that are to be plotted, provided that we have gnuplot.
        """
        if self.active:
            self.plot_2d_graphs()
            self.plot_3d_graphs()


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, format="%(message)s")
    parser = optparse.OptionParser("usage: %prog [options] [filenames]")
    options, args = parser.parse_args()

    if args:
        filenames = args
    else:
        parser.print_help()
        sys.exit(1)

    if len(args) > 2:
        parser.print_help()
        sys.exit(1)

    o = os.path.join(os.getcwd(),
                     "iozone-graphs-%s" % time.strftime('%Y-%m-%d-%H.%M.%S'))
    if not os.path.isdir(o):
        os.makedirs(o)

    a = IOzoneAnalyzer(list_files=filenames, output_dir=o)
    a.analyze()
    p = IOzonePlotter(results_file=filenames[0], output_dir=o)
    p.plot_all()



import os
import re
import hashlib
import logging

from aexpect import remote

from virttest import data_dir


class BuildError(Exception):

    def __init__(self, error_info):
        super(BuildError, self).__init__(error_info)
        self.error_info = error_info

    def __str__(self):
        e_msg = "Build Error: %s" % self.error_info
        return e_msg


class Builder(object):

    def __init__(self, params, address, source, shell_client=None,
                 shell_port=None, file_transfer_client=None,
                 file_transfer_port=None, username=None, password=None,
                 make_flags="", build_dir=None, build_dir_prefix=None,
                 shell_linesep=None, shell_prompt=None):
        """
        :param params: Dictionary with test parameters, used to get the default
                       values of all named parameters.
        :param address: Remote host or guest address
        :param source: Directory containing the source on the machine
                       where this script is running
        :param shell_client: The client to use ('ssh', 'telnet' or 'nc')
        :param shell_port: Port to connect to for the shell client
        :param file_transfer_client: The file transfer client to use ('scp' or
                                     'rss')
        :param file_transfer_port: Port to connect to for the file transfer
                                   client
        :param username: Username (if required)
        :param password: Password (if required)
        :param make_flags: Flags to pass to the make process, default: ""
        :param build_dir: Where to copy and build the files on target. If None,
                          use params['tmp_dir']
        :param build_dir_prefix: What to name the build directory on target
                                 If None, use the name of the source directory.
        :param shell_linesep: Line separator in the shell
        :param shell_prompt: Regexp that matches the prompt in the shell.
        """

        def full_build_path(build_dir, directory_prefix, make_flags):
            """
            Generates the full path for the build using the make flags and
            supplied build location.
            :return: The full path as a string
            """
            extra_flags_hash = hashlib.sha1()
            extra_flags_hash.update(make_flags)
            directory_name = "%s-%s" % (directory_prefix,
                                        (extra_flags_hash.hexdigest())[:8])
            return os.path.join(build_dir, directory_name)

        def def_helper(arg, param, default):
            if arg is None:
                return params.get(param, default)
            else:
                return arg

        self.address = address
        self.source = os.path.normpath(source)
        self.client = def_helper(shell_client, "shell_client", "ssh")
        self.port = def_helper(shell_port, "shell_port", "22")
        self.file_transfer_client = def_helper(file_transfer_client,
                                               "file_transfer_client", "scp")
        self.file_transfer_port = def_helper(file_transfer_port,
                                             "file_transfer_port", "22")
        self.username = def_helper(username, "username", "root")
        self.password = def_helper(password, "password", "redhat")
        self.make_flags = make_flags
        self.build_dir = def_helper(build_dir, "tmp_dir",
                                    data_dir.get_tmp_dir())
        if build_dir_prefix is None:
            build_dir_prefix = os.path.basename(source)
        self.full_build_path = full_build_path(self.build_dir,
                                               build_dir_prefix, make_flags)
        self.linesep = def_helper(shell_linesep, "shell_linesep", "\n")
        self.prompt = def_helper(shell_prompt, "shell_prompt",
                                 "^\[.*\][\#\$]\s*)$")

        self.session = remote.remote_login(self.client, self.address,
                                           self.port, self.username,
                                           self.password, self.prompt,
                                           self.linesep, timeout=360)

    def sync_directories(self):
        """
        Synchronize the directories between the local and remote machines
        :returns: True if any files needed to be copied; False otherwise. Does
        not support symlinks.
        """

        def get_local_hashes(path):
            """
            Create a dict of the hashes of all files in path on the local
            machine.
            :param path: Path to search
            """
            def hash_file(file_name):
                """
                Calculate hex-encoded hash of a file
                :param file_name: File to hash
                """
                f = open(file_name, mode='r')
                h = hashlib.sha1()
                while True:
                    buf = f.read(4096)
                    if not buf:
                        break
                    h.update(buf)
                return h.hexdigest()

            def visit(arg, dir_name, file_names):
                """
                Callback function to calculate and store hashes
                :param arg: Tuple with base path and the hash that will contain
                            the results.
                :param dir_name: Current directory
                :param file_names: File names in the current directory
                """
                (base_path, result) = arg
                for file_name in file_names:
                    path = os.path.join(dir_name, file_name)
                    if os.path.isfile(path):
                        result[
                            os.path.relpath(path, base_path)] = hash_file(path)

            result = {}
            os.path.walk(path, visit, (path, result))

            return result

        def get_remote_hashes(path, session, linesep):
            """
            Create a dict of the hashes of all files in path on the remote
            machine.
            :param path: Path to search
            :param session: Session object to use
            :param linesep: Line separation string for the remote system
            """

            cmd = 'test \! -d %s || find %s -type f | xargs sha1sum' % (path,
                                                                        path)
            status, output = session.cmd_status_output(cmd)
            if not status == 0:
                raise BuildError("Unable to get hashes of remote files: '%s'"
                                 % output)
            result = {}
            # Output is "<sum>  <filename><linesep><sum>  <filename>..."
            for line in output.split(linesep):
                if re.match("^[a-f0-9]{32,}  [^ ].*$", line):
                    (h, f) = line.split(None, 1)
                    result[os.path.relpath(f, path)] = h

            return result

        def list_recursive_dirnames(path):
            """
            List all directories that exist in path on the local machine
            :param path: Path to search
            """
            def visit(arg, dir_name, file_names):
                """
                Callback function list alla directories
                :param arg: Tuple with base path and the list that will contain
                            the results.
                :param dir_name: Current directory
                :param file_names: File names in the current directory
                """
                (base_path, result) = arg
                for file_name in file_names:
                    path = os.path.join(dir_name, file_name)
                    if os.path.isdir(path):
                        result.append(os.path.relpath(path, base_path))

            result = []
            os.path.walk(path, visit, (path, result))

            return result

        remote_hashes = get_remote_hashes(self.full_build_path, self.session,
                                          self.linesep)
        local_hashes = get_local_hashes(self.source)

        to_transfer = []
        for rel_path in list(local_hashes.keys()):
            rhash = remote_hashes.get(rel_path)
            if rhash is None or not rhash == local_hashes[rel_path]:
                to_transfer.append(rel_path)

        need_build = False
        if to_transfer:
            logging.info("Need to copy files to %s on target" %
                         self.full_build_path)
            need_build = True

            # Create all directories
            dirs = list_recursive_dirnames(self.source)
            if dirs:
                dirs_text = " ".join(dirs)
                fmt_arg = (self.full_build_path, self.full_build_path,
                           dirs_text)
                cmd = 'mkdir -p %s && cd %s && mkdir -p %s' % fmt_arg
            else:
                cmd = 'mkdir -p %s' % self.full_build_path
            status, output = self.session.cmd_status_output(cmd)
            if not status == 0:
                raise BuildError("Unable to create remote directories: '%s'"
                                 % output)

            # Copy files
            for file_name in to_transfer:
                local_path = os.path.join(self.source, file_name)
                remote_path = os.path.join(self.full_build_path, file_name)
                remote.copy_files_to(self.address, self.file_transfer_client,
                                     self.username, self.password,
                                     self.file_transfer_port, local_path,
                                     remote_path)

        else:
            logging.info("Directory %s on target already up-to-date" %
                         self.full_build_path)

        return need_build

    def make(self):
        """
        Execute make on the remote system
        """
        logging.info("Building in %s on target" % self.full_build_path)
        cmd = 'make -C %s %s' % (self.full_build_path, self.make_flags)
        status, output = self.session.cmd_status_output(cmd)
        if not status == 0:
            raise BuildError("Unable to make: '%s'" % output)

    def build(self):
        """
        Synchronize all files and execute 'make' on the remote system if
        needed.
        :returns: The path to the build directory on the remote machine
        """
        if self.sync_directories():
            self.make()

        return self.full_build_path



"""
Classes and functions to handle block/disk images for libvirt.

This exports:
  - two functions for get image/blkdebug filename
  - class for image operates and basic parameters
  - class for storage pool operations
"""

import re
import logging

from avocado.utils import process

from virttest import storage
from virttest import virsh


class QemuImg(storage.QemuImg):

    """
    libvirt class for handling operations of disk/block images.
    """

    def __init__(self, params, root_dir, tag):
        """
        Init the default value for image object.

        :param params: Dictionary containing the test parameters.
        :param root_dir: Base directory for relative filenames.
        :param tag: Image tag defined in parameter images.
        """
        storage.QemuImg(params, root_dir, tag)
        # Please init image_cmd for libvirt in this class
        # self.image_cmd =

    def create(self, params):
        """
        Create an image.

        :param params: Dictionary containing the test parameters.

        :note: params should contain:
        """
        raise NotImplementedError

    def convert(self, params, root_dir):
        """
        Convert image

        :param params: A dict
        :param root_dir: dir for save the convert image

        :note: params should contain:
        """
        raise NotImplementedError

    def rebase(self, params):
        """
        Rebase image

        :param params: A dict

        :note: params should contain:
        """
        raise NotImplementedError

    def commit(self):
        """
        Commit image to it's base file
        """
        raise NotImplementedError

    def snapshot_create(self):
        """
        Create a snapshot image.

        :note: params should contain:
        """
        raise NotImplementedError

    def snapshot_del(self, blkdebug_cfg=""):
        """
        Delete a snapshot image.

        :param blkdebug_cfg: The configure file of blkdebug

        :note: params should contain:
               snapshot_image_name -- the name of snapshot image file
        """
        raise NotImplementedError

    def remove(self):
        """
        Remove an image file.

        :note: params should contain:
        """
        raise NotImplementedError

    def check_image(self, params, root_dir):
        """
        Check an image using the appropriate tools for each virt backend.

        :param params: Dictionary containing the test parameters.
        :param root_dir: Base directory for relative filenames.

        :note: params should contain:

        :raise VMImageCheckError: In case qemu-img check fails on the image.
        """
        raise NotImplementedError


class StoragePool(object):

    """
    Pool Manager for libvirt storage with virsh commands
    """

    def __init__(self, virsh_instance=virsh):
        # An instance of Virsh class
        # Help to setup connection to virsh instance
        self.virsh_instance = virsh_instance

    def list_pools(self):
        """
        Return a dict include pools' information with structure:
            pool_name ==> pool_details(a dict: feature ==> value)
        """
        # Allow it raise exception if command has executed failed.
        result = self.virsh_instance.pool_list("--all", ignore_status=False)
        pools = {}
        lines = result.stdout_text.strip().splitlines()
        if len(lines) > 2:
            head = lines[0]
            lines = lines[2:]
        else:
            return pools

        for line in lines:
            details = line.split()
            details_dict = {}
            head_iter = enumerate(head.split())
            while True:
                try:
                    (index, column) = next(head_iter)
                except StopIteration:
                    break
                if re.match("[N|n]ame", column):
                    pool_name = details[index]
                else:
                    details_dict[column] = details[index]
            pools[pool_name] = details_dict
        return pools

    def pool_exists(self, name):
        """
        Check whether pool exists on given libvirt
        """
        try:
            pools = self.list_pools()
        except process.CmdError:
            return False

        return name in pools

    def pool_state(self, name):
        """
        Get pool's state.

        :return: active/inactive, and None when something wrong.
        """
        try:
            return self.list_pools()[name]['State']
        except (process.CmdError, KeyError):
            return None

    def pool_autostart(self, name):
        """
        Get pool's autostart.

        :return: yes/no, and None when something wrong.
        """
        try:
            return self.list_pools()[name]['Autostart']
        except (process.CmdError, KeyError):
            return None

    def pool_info(self, name):
        """
        Get pool's information.

        :return: A dict include pool's information:
                Name ==> value
                UUID ==> value
        """
        info = {}
        try:
            result = self.virsh_instance.pool_info(name, ignore_status=False)
        except process.CmdError:
            return info

        for line in result.stdout_text.splitlines():
            params = line.split(':')
            if len(params) == 2:
                name = params[0].strip()
                value = params[1].strip()
                info[name] = value
        return info

    def get_pool_uuid(self, name):
        """
        Get pool's uuid.

        :return: Pool uuid.
        """
        return self.pool_info(name)["UUID"]

    def is_pool_active(self, name):
        """
        Check whether pool is active on given libvirt
        """
        if self.pool_state(name) == "active":
            return True
        return False

    def is_pool_persistent(self, name):
        """
        Check whether pool is persistent
        """
        if self.pool_info(name)["Persistent"] == "yes":
            return True
        return False

    def delete_pool(self, name):
        """
        Destroy and Delete a pool if it exists on given libvirt

        It's reasonable to delete a pool by calling pool-delete.
        However, due to pool-delete operation is non-recoverable.
        Redhat suggests to achieve this objective by virsh,
        1) virsh pool-destroy pool-name
        2) virsh pool-undefine pool-name

        Please refer to the following URI for more details.
        https://access.redhat.com/documentation/en-US
        /Red_Hat_Enterprise_Linux/6/html
        /Virtualization_Administration_Guide
        /chap-Virtualization_Administration_Guide
        -Storage_Pools-Storage_Pools.html#delete-ded-disk-storage-pool
        """
        if self.is_pool_active(name):
            if not self.virsh_instance.pool_destroy(name):
                # TODO: Allow pool_destroy to raise exception.
                #       Because some testcase rely on this function,
                #       I should start this work after this module is accepted.
                logging.error("Destroy pool '%s' failed.", name)
                return False

        # Undefine pool anyway
        try:
            self.virsh_instance.pool_undefine(name, ignore_status=False)
        except process.CmdError as detail:
            if self.pool_exists(name):
                logging.error("Undefine pool '%s' failed:%s", name, detail)
                return False
        logging.info("Deleted pool '%s'", name)
        return True

    def set_pool_autostart(self, name, extra=""):
        """
        Set given pool as autostart
        """
        try:
            self.virsh_instance.pool_autostart(name, extra, ignore_status=False)
        except process.CmdError:
            logging.error("Autostart pool '%s' failed.", name)
            return False
        logging.info("Set pool '%s' autostart.", name)
        return True

    def build_pool(self, name, options="", **dargs):
        """
        Build pool.
        """
        try:
            self.virsh_instance.pool_build(name, options, **dargs)
        except process.CmdError:
            logging.error("Build pool '%s' failed.", name)
            return False
        logging.info("Built pool '%s'", name)
        return True

    def start_pool(self, name):
        """
        Start pool if it is inactive.
        """
        if self.is_pool_active(name):
            logging.info("Pool '%s' is already active.", name)
            return True
        try:
            self.virsh_instance.pool_start(name, ignore_status=False)
        except process.CmdError as details:
            logging.error("Start pool '%s' failed: %s", name, details)
            return False
        logging.info("Started pool '%s'", name)
        return True

    def destroy_pool(self, name):
        """
        Destroy pool if it is active.
        """
        if not self.is_pool_active(name):
            logging.info("pool '%s' is already inactive.", name)
            return True
        return self.virsh_instance.pool_destroy(name)

    def define_dir_pool(self, name, target_path):
        """
        Define a directory type pool.
        """
        try:
            self.virsh_instance.pool_define_as(name, "dir", target_path,
                                               ignore_status=False)
        except process.CmdError:
            logging.error("Define dir pool '%s' failed.", name)
            return False
        logging.info("Defined pool '%s'", name)
        return True

    def define_fs_pool(self, name, block_device, target_path):
        """
        Define a filesystem type pool.
        """
        try:
            self.virsh_instance.pool_define_as(name, "fs", target_path,
                                               extra="--source-dev %s" % block_device,
                                               ignore_status=False)
        except process.CmdError:
            logging.error("Define fs pool '%s' failed.", name)
            return False
        logging.info("Defined pool '%s'", name)
        return True

    def define_lvm_pool(self, name, block_device, vg_name, target_path):
        """
        Define a lvm type pool.
        """
        try:
            extra = "--source-dev %s --source-name %s" % (block_device,
                                                          vg_name)
            self.virsh_instance.pool_define_as(name, "logical", target_path,
                                               extra, ignore_status=False)
        except process.CmdError:
            logging.error("Define logic pool '%s' failed.", name)
            return False
        logging.info("Defined pool '%s'", name)
        return True

    def define_disk_pool(self, name, block_device, target_path):
        """
        Define a disk type pool.
        """
        try:
            extra = "--source-dev %s" % block_device
            self.virsh_instance.pool_define_as(name, "disk", target_path,
                                               extra, ignore_status=False)
        except process.CmdError:
            logging.error("Define disk pool '%s' failed.", name)
            return False
        logging.info("Defined pool '%s'", name)
        return True

    def define_iscsi_pool(self, name, source_host, source_dev, target_path):
        """
        Define a iscsi type pool.
        """
        try:
            extra = "--source-host %s  --source-dev %s" % (source_host,
                                                           source_dev)
            self.virsh_instance.pool_define_as(name, "iscsi", target_path,
                                               extra, ignore_status=False)
        except process.CmdError:
            logging.error("Define iscsi pool '%s' failed.", name)
            return False
        logging.info("Define pool '%s'", name)
        return True

    def define_netfs_pool(self, name, source_host, source_path, target_path):
        """
        Define a netfs type pool.
        """
        try:
            extra = "--source-host %s --source-path %s" % (source_host,
                                                           target_path)
            self.virsh_instance.pool_define_as(name, "netfs", target_path,
                                               extra, ignore_status=False)
        except process.CmdError:
            logging.error("Define netfs pool '%s' failed.", name)
            return False
        logging.info("Define pool '%s'", name)
        return True

    def define_rbd_pool(self, name, source_host, source_name, extra=""):
        """
        Define a rbd type pool.
        """
        try:
            extra = ("--source-host %s --source-name %s %s" %
                     (source_host, source_name, extra))
            self.virsh_instance.pool_define_as(name, "rbd", "",
                                               extra, ignore_status=False)
        except process.CmdError:
            logging.error("Define rbd pool '%s' failed.", name)
            return False
        logging.info("Define pool '%s'", name)
        return True


class PoolVolume(object):

    """Volume Manager for libvirt storage pool."""

    def __init__(self, pool_name, virsh_instance=virsh):
        self.pool_name = pool_name
        self.virsh_instance = virsh_instance

    def list_volumes(self):
        """
        Return a dict include volumes' name(key) and path(value).
        """
        volumes = {}
        try:
            result = self.virsh_instance.vol_list(self.pool_name,
                                                  ignore_status=False)
        except process.CmdError as detail:
            logging.error('List volume failed: %s', detail)
            return volumes

        lines = result.stdout_text.strip().splitlines()
        if len(lines) > 2:
            head = lines[0]
            lines = lines[2:]
        else:
            return volumes

        for line in lines:
            # Path may be not standard unix path
            try:
                path = re.findall("\s+\S*/.*", line)[0]
            except IndexError:
                # Do not find a path
                path = ""
            name = line.split(path)[0].lstrip()
            volumes[name] = path.strip()
        return volumes

    def volume_exists(self, name):
        volumes = self.list_volumes()
        return name in volumes

    def volume_info(self, name):
        """
        Get volume's information with command vol-info.
        """
        info = {}
        try:
            result = self.virsh_instance.vol_info(name, self.pool_name,
                                                  ignore_status=False)
        except process.CmdError as detail:
            logging.error("Get volume information failed: %s", detail)
            return info

        for line in result.stdout_text.strip().splitlines():
            attr = line.split(':')[0]
            value = line.split("%s:" % attr)[-1].strip()
            info[attr] = value
        return info

    def create_volume(self, name, capability,
                      allocation=None, frmt=None):
        """
        Create a volume in pool.
        """
        if self.volume_exists(name):
            logging.debug("Volume '%s' already exists.", name)
            return False
        try:
            self.virsh_instance.vol_create_as(name, self.pool_name,
                                              capability, allocation, frmt,
                                              ignore_status=False, debug=True)
        except process.CmdError as detail:
            logging.error("Create volume failed:%s", detail)
            return False

        if not self.volume_exists(name):
            logging.error("Created volume does not exist:%s",
                          self.list_volumes())
            return False
        return True

    def delete_volume(self, name):
        """
        Remove a volume.
        """
        if self.volume_exists(name):
            try:
                self.virsh_instance.vol_delete(name, self.pool_name,
                                               ignore_status=False)
            except process.CmdError as detail:
                logging.error("Delete volume failed:%s", detail)
                return False
            if not self.volume_exists(name):
                logging.debug("Volume '%s' has been deleted.", name)
                return True
            else:
                logging.debug("Delete volume '%s' failed.", name)
                return False
        else:
            logging.info("Volume '%s' does not exist.", name)
            return True     # Return True for expected result

    def clone_volume(self, old_name, new_name):
        """
        Clone a volume
        """
        if self.volume_exists(old_name) and not self.volume_exists(new_name):
            try:
                self.virsh_instance.vol_clone(old_name, new_name,
                                              self.pool_name,
                                              ignore_status=False)
            except process.CmdError as detail:
                logging.error("Clone volume failed:%s", detail)
                return False
            if self.volume_exists(new_name):
                logging.debug("Volume '%s' has been created by clone.",
                              new_name)
                return True
            else:
                logging.debug("Volume '%s' clone failed.", old_name)
                return False
        else:
            logging.info("Volume '%s' does not exist or '%s' has been exist."
                         % (old_name, new_name))
            return False


def check_qemu_image_lock_support():
    """
    QEMU commit 244a566(qemu-2.10) introduced the image locking feature which brought
    a new option '-U' to the qemu-img command:info, compare, check, bench, convert,
    dd, map, snapshot, rebase. This method provides one way to determine whether
    current qemu-img support or not.

    :return: True if current qemu-img command support
    """
    cmd = "qemu-img"
    try:
        binary_path = process.run("which %s" % cmd).stdout_text.strip()
    except process.CmdError:
        raise process.CmdError(cmd, binary_path,
                               "qemu-img command is not found")
    cmd_result = process.run(binary_path + ' -h', ignore_status=True,
                             shell=True, verbose=False)
    return '-U' in cmd_result.stdout_text



"""
interface device support class(es)

http://libvirt.org/formatdomain.html#elementsNICS
http://libvirt.org/formatnwfilter.html#nwfconceptsvars
"""

from virttest.libvirt_xml import accessors
from virttest.libvirt_xml import xcepts
from virttest.libvirt_xml.devices import base, librarian


class Interface(base.TypedDeviceBase):

    __slots__ = ('source', 'hostdev_address', 'managed', 'mac_address',
                 'bandwidth', 'model', 'coalesce', 'link_state', 'target', 'driver',
                 'address', 'boot', 'rom', 'mtu', 'filterref', 'backend',
                 'virtualport_type', 'alias', "ips", "teaming", "vlan", "port")

    def __init__(self, type_name, virsh_instance=base.base.virsh):
        super(Interface, self).__init__(device_tag='interface',
                                        type_name=type_name,
                                        virsh_instance=virsh_instance)
        accessors.XMLElementDict(property_name="source",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='source')
        accessors.XMLElementNest('hostdev_address', self, parent_xpath='/source',
                                 tag_name='address', subclass=self.Address,
                                 subclass_dargs={'type_name': 'pci',
                                                 'virsh_instance': virsh_instance})
        accessors.XMLAttribute(property_name="managed",
                               libvirtxml=self,
                               forbidden=None,
                               parent_xpath='/',
                               tag_name='interface',
                               attribute='managed')
        accessors.XMLElementDict(property_name="target",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='target')
        accessors.XMLElementDict(property_name="backend",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='backend')
        accessors.XMLAttribute(property_name="mac_address",
                               libvirtxml=self,
                               forbidden=None,
                               parent_xpath='/',
                               tag_name='mac',
                               attribute='address')
        accessors.XMLAttribute(property_name="link_state",
                               libvirtxml=self,
                               forbidden=None,
                               parent_xpath='/',
                               tag_name='link',
                               attribute='state')
        accessors.XMLAttribute(property_name="boot",
                               libvirtxml=self,
                               forbidden=None,
                               parent_xpath='/',
                               tag_name='boot',
                               attribute='order')
        accessors.XMLElementNest("bandwidth", self,
                                 parent_xpath='/',
                                 tag_name='bandwidth',
                                 subclass=self.Bandwidth,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementNest("driver", self,
                                 parent_xpath='/',
                                 tag_name='driver',
                                 subclass=self.Driver,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementNest("filterref", self,
                                 parent_xpath='/',
                                 tag_name='filterref',
                                 subclass=self.Filterref,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLAttribute(property_name="model",
                               libvirtxml=self,
                               forbidden=None,
                               parent_xpath='/',
                               tag_name='model',
                               attribute='type')
        accessors.XMLElementDict(property_name="coalesce",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/coalesce/rx',
                                 tag_name='frames')
        accessors.XMLElementDict(property_name="rom",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='rom')
        accessors.XMLElementDict(property_name="mtu",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='mtu')
        accessors.XMLElementNest('address', self, parent_xpath='/',
                                 tag_name='address', subclass=self.Address,
                                 subclass_dargs={'type_name': 'pci',
                                                 'virsh_instance': virsh_instance})
        accessors.XMLAttribute('virtualport_type', self, parent_xpath='/',
                               tag_name='virtualport', attribute='type')
        accessors.XMLElementDict('alias', self, parent_xpath='/',
                                 tag_name='alias')
        accessors.XMLElementList(property_name='ips',
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 marshal_from=self.marshal_from_ips,
                                 marshal_to=self.marshal_to_ips)
        accessors.XMLElementDict(property_name="teaming",
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='teaming')
        accessors.XMLElementNest("vlan", self,
                                 parent_xpath='/',
                                 tag_name='vlan',
                                 subclass=self.Vlan,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementDict(property_name='port',
                                 libvirtxml=self,
                                 forbidden=None,
                                 parent_xpath='/',
                                 tag_name='port')

    # For convenience
    Address = librarian.get('address')

    Filterref = librarian.get('filterref')

    @staticmethod
    def marshal_from_ips(item, index, libvirtxml):
        """Convert an Address instance into tag + attributes"""
        """Convert a dictionary into a tag + attributes"""
        del index           # not used
        del libvirtxml      # not used
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of ip"
                                         "attributes, not a %s"
                                         % str(item))
        return ('ip', dict(item))  # return copy of dict, not reference

    @staticmethod
    def marshal_to_ips(tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into an Address instance """
        del index                    # not used
        del libvirtxml               # not used
        if not tag == 'ip':
            return None              # skip this one
        return dict(attr_dict)       # return copy of dict, not reference

    def new_bandwidth(self, **dargs):
        """
        Return a new interafce banwidth instance from dargs
        """
        new_one = self.Bandwidth(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def new_driver(self, **dargs):
        """
        Return a new interafce driver instance from dargs
        """
        new_one = self.Driver(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def new_iface_address(self, **dargs):
        """
        Return a new interface Address instance and set properties from dargs
        """
        new_one = self.Address("pci", virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def new_filterref(self, **dargs):
        """
        Return a new interafce filterref instance from dargs
        """
        new_one = self.Filterref(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def new_vlan(self, **dargs):
        """
        This defines a new element vlan in the interface xml.

        :param dargs: a dict with keys as "trunk" and "tags",
        like {'trunk': 'yes', 'tags': [{'id': '42'}, {'id': '123', 'nativeMode': 'untagged'}]}
        :Return: a new vlan instance from dargs
        """
        new_one = self.Vlan(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    class Bandwidth(base.base.LibvirtXMLBase):

        """
        Interface bandwidth xml class.

        Properties:

        inbound:
            dict. Keys: average, peak, floor, burst
        outbound:
            dict. Keys: average, peak, floor, burst
        """
        __slots__ = ("inbound", "outbound")

        def __init__(self, virsh_instance=base.base.virsh):
            accessors.XMLElementDict("inbound", self, parent_xpath="/",
                                     tag_name="inbound")
            accessors.XMLElementDict("outbound", self, parent_xpath="/",
                                     tag_name="outbound")
            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<bandwidth/>'

    class Driver(base.base.LibvirtXMLBase):

        """
        Interface Driver xml class.

        Properties:

        driver:
            dict.
        host:
            dict. Keys: csum, gso, tso4, tso6, ecn, ufo
        guest:
            dict. Keys: csum, gso, tso4, tso6, ecn, ufo
        """
        __slots__ = ("driver_attr", "driver_host", "driver_guest")

        def __init__(self, virsh_instance=base.base.virsh):
            accessors.XMLElementDict("driver_attr", self, parent_xpath="/",
                                     tag_name="driver")
            accessors.XMLElementDict("driver_host", self, parent_xpath="/",
                                     tag_name="host")
            accessors.XMLElementDict("driver_guest", self, parent_xpath="/",
                                     tag_name="guest")
            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<driver/>'

    class Vlan(base.base.LibvirtXMLBase):

        """
        Interface vlan xml class.

        Properties:

        trunk:
            attribute.
        tags:
            list. tags element dict list
         """
        __slots__ = ("trunk", "tags")

        def __init__(self, virsh_instance=base.base.virsh):
            accessors.XMLAttribute(property_name="trunk",
                                   libvirtxml=self,
                                   forbidden=None,
                                   parent_xpath='/',
                                   tag_name='vlan',
                                   attribute='trunk')
            accessors.XMLElementList(property_name='tags',
                                     libvirtxml=self,
                                     parent_xpath='/',
                                     marshal_from=self.marshal_from_tag,
                                     marshal_to=self.marshal_to_tag)
            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<vlan/>'

        @staticmethod
        def marshal_from_tag(item, index, libvirtxml):
            """Convert a dictionary into a tag + attributes"""
            del index  # not used
            del libvirtxml  # not used
            if not isinstance(item, dict):
                raise xcepts.LibvirtXMLError("Expected a dictionary of tag "
                                             "attributes, not a %s"
                                             % str(item))
                # return copy of dict, not reference
            return ('tag', dict(item))

        @staticmethod
        def marshal_to_tag(tag, attr_dict, index, libvirtxml):
            """Convert a tag + attributes into a dictionary"""
            del index  # not used
            del libvirtxml  # not used
            if tag != 'tag':
                return None  # skip this one
            return dict(attr_dict)  # return copy of dict, not reference@
